@@ -15,7 +15,7 @@
 //
 // See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
-// HitagS emulation (preliminary test version)
+// Hitag S emulation (preliminary test version)
 //-----------------------------------------------------------------------------
 
 #include "hitagS.h"
@@ -32,6 +32,7 @@
 #include "hitag2/hitag2_crypto.h"
 #include "lfadc.h"
 #include "crc.h"
+#include <protocols.h>
 
 #define CRC_PRESET 0xFF
 #define CRC_POLYNOM 0x1D
@@ -75,25 +76,30 @@ static uint32_t rnd = 0x74124485;   // random number
 #define ht2bs_4b(a,b,c,d)   (~(((d|c)&(a^b))^(d|a|b)))
 #define ht2bs_5c(a,b,c,d,e) (~((((((c^e)|d)&a)^b)&(c^b))^(((d^e)|a)&((d^b)|c))))
 
-// Sam7s has several timers, we will use the source TIMER_CLOCK1 (aka AT91C_TC_CLKS_TIMER_DIV1_CLOCK)
-// TIMER_CLOCK1 = MCK/2, MCK is running at 48 MHz, Timer is running at 48/2 = 24 MHz
+// Sam7s has several timers, we will use the source TIMER_CLOCK3 (aka AT91C_TC_CLKS_TIMER_DIV3_CLOCK)
+// TIMER_CLOCK3 = MCK/32, MCK is running at 48 MHz, Timer is running at 48MHz/32 = 1500 KHz
 // Hitag units (T0) have duration of 8 microseconds (us), which is 1/125000 per second (carrier)
-// T0 = TIMER_CLOCK1 / 125000 = 192
-#ifndef T0
-#define T0                             192
-#endif
+// T0 = TIMER_CLOCK3 / 125000 = 12
+
+#define T0                             12
 
 #define HITAG_FRAME_LEN                20
+
+// TC0 and TC1 are 16-bit counters and will overflow after 5461 * T0
+// Ensure not to set these timings above 5461 (~43ms) when comparing without considering overflow, as they will never reach that value.
+
 #define HITAG_T_STOP                   36  /* T_EOF should be > 36 */
 #define HITAG_T_LOW                    8   /* T_LOW should be 4..10 */
 #define HITAG_T_0_MIN                  15  /* T[0] should be 18..22 */
 #define HITAG_T_1_MIN                  25  /* T[1] should be 26..30 */
-//#define HITAG_T_EOF   40 /* T_EOF should be > 36 */
+#define HITAG_T_0                      20  /* T[0] should be 18..22 */
+#define HITAG_T_1                      28  /* T[1] should be 26..30 */
+// #define HITAG_T_EOF   40 /* T_EOF should be > 36 */
 #define HITAG_T_EOF                    80   /* T_EOF should be > 36 */
-#define HITAG_T_WAIT_1                 200  /* T_wresp should be 199..206 */
-#define HITAG_T_WAIT_2                 90   /* T_wresp should be 199..206 */
-#define HITAG_T_WAIT_MAX               300  /* bit more than HITAG_T_WAIT_1 + HITAG_T_WAIT_2 */
-#define HITAG_T_PROG_MAX               750
+#define HITAG_T_WAIT_RESP              200  /* T_wresp should be 204..212 */
+#define HITAG_T_WAIT_SC                200   /* T_wsc should be 90..5000 */
+#define HITAG_T_WAIT_FIRST             300  /* T_wfc should be 280..565 (T_ttf) */
+#define HITAG_T_PROG_MAX               750  /* T_prog should be 716..726 */
 
 #define HITAG_T_TAG_ONE_HALF_PERIOD    10
 #define HITAG_T_TAG_TWO_HALF_PERIOD    25
@@ -272,6 +278,7 @@ static void hitag_reader_send_bit(int bit, bool ledcontrol) {
     if (ledcontrol) LED_A_ON();
     // Reset clock for the next bit
     AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
+    while (AT91C_BASE_TC0->TC_CV > 0);
 
     // Binary puls length modulation (BPLM) is used to encode the data stream
     // This means that a transmission of a one takes longer than that of a zero
@@ -293,16 +300,16 @@ static void hitag_reader_send_bit(int bit, bool ledcontrol) {
     }
 #else
     // Wait for 4-10 times the carrier period
-    while (AT91C_BASE_TC0->TC_CV < T0 * 6) {};
+    while (AT91C_BASE_TC0->TC_CV < T0 * HITAG_T_LOW) {};
 
     LOW(GPIO_SSC_DOUT);
 
     if (bit == 0) {
         // Zero bit: |_-|
-        while (AT91C_BASE_TC0->TC_CV < T0 * 22) {};
+        while (AT91C_BASE_TC0->TC_CV < T0 * HITAG_T_0) {};
     } else {
         // One bit: |_--|
-        while (AT91C_BASE_TC0->TC_CV < T0 * 28) {};
+        while (AT91C_BASE_TC0->TC_CV < T0 * HITAG_T_1) {};
     }
 #endif
 
@@ -319,11 +326,11 @@ static void hitag_reader_send_frame(const uint8_t *frame, size_t frame_len, bool
     }
     // send EOF
     AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
-
+    while (AT91C_BASE_TC0->TC_CV > 0);
     HIGH(GPIO_SSC_DOUT);
 
     // Wait for 4-10 times the carrier period
-    while (AT91C_BASE_TC0->TC_CV < T0 * 6) {};
+    while (AT91C_BASE_TC0->TC_CV < T0 * HITAG_T_LOW) {};
 
     LOW(GPIO_SSC_DOUT);
 }
@@ -331,8 +338,8 @@ static void hitag_reader_send_frame(const uint8_t *frame, size_t frame_len, bool
 static void hitagS_init_clock(void) {
 
     // Enable Peripheral Clock for
-    //   TIMER_CLOCK0, used to measure exact timing before answering
-    //   TIMER_CLOCK1, used to capture edges of the tag frames
+    //   Timer Counter 0, used to measure exact timing before answering
+    //   Timer Counter 1, used to capture edges of the tag frames
     AT91C_BASE_PMC->PMC_PCER |= (1 << AT91C_ID_TC0) | (1 << AT91C_ID_TC1);
 
     AT91C_BASE_PIOA->PIO_BSR = GPIO_SSC_FRAME;
@@ -341,35 +348,34 @@ static void hitagS_init_clock(void) {
     AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
     AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
 
-    // TC0: Capture mode, default timer source = MCK/2 (TIMER_CLOCK1), no triggers
-    AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK;
+    // TC0: Capture mode, clock source = MCK/32 (TIMER_CLOCK3), no triggers
+    AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK;
 
-    // TC1: Capture mode, default timer source = MCK/2 (TIMER_CLOCK1), TIOA is external trigger,
-    // external trigger rising edge, load RA on falling edge of TIOA.
+    // TC1: Capture mode, clock source = MCK/32 (TIMER_CLOCK3), TIOA is external trigger,
+    // external trigger falling edge, set RA on falling edge of TIOA.
     AT91C_BASE_TC1->TC_CMR =
-        AT91C_TC_CLKS_TIMER_DIV1_CLOCK |
-        AT91C_TC_ETRGEDG_FALLING |
-        AT91C_TC_ABETRG |
-        AT91C_TC_LDRA_FALLING |
-        AT91C_TC_ACPA_CLEAR | // RA comperator clears TIOA (carry bit)
-        AT91C_TC_ASWTRG_SET;  // SWTriger sets TIOA (carry bit)
+        AT91C_TC_CLKS_TIMER_DIV3_CLOCK |
+        AT91C_TC_ETRGEDG_FALLING | // external trigger on falling edge
+        AT91C_TC_ABETRG |          // TIOA is used as an external trigger.
+        AT91C_TC_LDRA_FALLING |    // load RA on on falling edge
+        AT91C_TC_ACPA_CLEAR |      // RA comperator clears TIOA (carry bit)
+        AT91C_TC_ASWTRG_SET;       // SWTriger sets TIOA (carry bit)
 
-    AT91C_BASE_TC0->TC_RC  = 0; // set TIOA (carry bit) on overflow, return to zero
-    AT91C_BASE_TC0->TC_RA  = 1; // clear carry bit on next clock cycle
+    AT91C_BASE_TC1->TC_RA  = 1; // clear carry bit on next clock cycle
 
     // Enable and reset counters
     AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN | AT91C_TC_SWTRG;
     AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKEN | AT91C_TC_SWTRG;
 
+    // for (size_t i = 0; i < 10; i++) __asm("");
+    // uint16_t cv0 = AT91C_BASE_TC0->TC_CV;
+
     // synchronized startup procedure
-    while (AT91C_BASE_TC0->TC_CV > 0); // wait until TC0 returned to zero
-//    while (AT91C_BASE_TC0->TC_CV < 2); // and has started (TC_CV > TC_RA, now TC1 is cleared)
-
-    // return to zero
-    AT91C_BASE_TC1->TC_CCR = AT91C_TC_SWTRG;
-    AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
-    while (AT91C_BASE_TC0->TC_CV > 0);
+    // In theory, with MCK/32, we shouldn't be waiting longer than 32 instruction statements, right?
+    while (AT91C_BASE_TC0->TC_CV > 0) {}; // wait until TC0 returned to zero
+//    while (AT91C_BASE_TC0->TC_CV < 2) {}; // and has started (TC_CV > TC_RA, now TC1 is cleared)
 
+    // Dbprintf("TC0_CV0:%i TC0_CV:%i TC1_CV:%i", cv0, AT91C_BASE_TC0->TC_CV, AT91C_BASE_TC1->TC_CV);
 }
 
 static void hitagS_stop_clock(void) {
@@ -450,7 +456,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
 
             tag.pstate = HT_READY;
             tag.tstate = HT_NO_OP;
-            if ((rx[0] & 0xf0) == 0x30) {
+            if ((rx[0] & 0xf0) == HITAGS_UID_REQ_STD) {
                 if (g_dbglevel >= DBG_EXTENDED)
                     Dbprintf("HT_STANDARD");
 
@@ -458,7 +464,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                 sof_bits = 1;
                 m = AC2K;
             }
-            if ((rx[0] & 0xf0) == 0xc0) {
+            if ((rx[0] & 0xf0) == HITAGS_UID_REQ_ADV) {
                 tag.mode = HT_ADVANCED;
                 if (g_dbglevel >= DBG_EXTENDED)
                     Dbprintf("HT_ADVANCED");
@@ -467,7 +473,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                 m = AC2K;
             }
 
-            if ((rx[0] & 0xf0) == 0xd0) {
+            if ((rx[0] & 0xf0) == HITAGS_UID_REQ_FADV) {
                 if (g_dbglevel >= DBG_EXTENDED)
                     Dbprintf("HT_FAST_ADVANCED");
 
@@ -488,7 +494,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                 DbpString("SELECT");
             }
 
-            if (check_select(rx, tag.uid) == 1) {
+            if ((rx[0] & 0xf8) == HITAGS_SELECT && check_select(rx, tag.uid) == 1) {
                 if (g_dbglevel >= DBG_EXTENDED) {
                     DbpString("SELECT match");
                 }
@@ -576,7 +582,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
         }
         case 40: {
             if (g_dbglevel >= DBG_EXTENDED)
-                Dbprintf("WRITE");
+                Dbprintf("WRITE DATA");
             //data received to be written
             if (tag.tstate == HT_WRITING_PAGE_DATA) {
                 tag.tstate = HT_NO_OP;
@@ -611,7 +617,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
         }
         case 20: {
             //write page, write block, read page or read block command received
-            if ((rx[0] & 0xf0) == 0xc0) { //read page
+            if ((rx[0] & 0xf0) == HITAGS_READ_PAGE) { //read page
                 //send page data
                 uint8_t page = ((rx[0] & 0x0f) * 16) + ((rx[1] & 0xf0) / 16);
                 *txlen = 32;
@@ -642,7 +648,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                     *txlen = 0;
                 }
 
-            } else if ((rx[0] & 0xf0) == 0xd0) { //read block
+            } else if ((rx[0] & 0xf0) == HITAGS_READ_BLOCK) { //read block
 
                 uint8_t page = ((rx[0] & 0x0f) * 16) + ((rx[1] & 0xf0) / 16);
                 *txlen = 32 * 4;
@@ -672,7 +678,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                     *txlen = 0;
                 }
 
-            } else if ((rx[0] & 0xf0) == 0x80) { //write page
+            } else if ((rx[0] & 0xf0) == HITAGS_WRITE_PAGE) { //write page
 
                 uint8_t page = ((rx[0] & 0x0f) * 16) + ((rx[1] & 0xf0) / 16);
 
@@ -688,7 +694,7 @@ static void hitagS_handle_reader_command(uint8_t *rx, const size_t rxlen,
                     tag.tstate = HT_WRITING_PAGE_DATA;
                 }
 
-            } else if ((rx[0] & 0xf0) == 0x90) { //write block
+            } else if ((rx[0] & 0xf0) == HITAGS_WRITE_BLOCK) { //write block
 
                 uint8_t page = ((rx[0] & 0x0f) * 6) + ((rx[1] & 0xf0) / 16);
                 hitagS_set_frame_modulation();
@@ -741,7 +747,7 @@ void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontr
     set_tracing(true);
     clear_trace();
 
-    DbpString("Starting HitagS simulation");
+    DbpString("Starting Hitag S simulation");
     if (ledcontrol) LED_D_ON();
 
     tag.pstate = HT_READY;
@@ -756,7 +762,7 @@ void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontr
             }
         }
 
-        DbpString("Loading hitagS memory...");
+        DbpString("Loading hitag S memory...");
         memcpy((uint8_t *)tag.pages, data, 4 * 64);
     } else {
         // use the last read tag
@@ -870,8 +876,8 @@ void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontr
     LOW(GPIO_SSC_DOUT);
 
     // Enable Peripheral Clock for
-    //   TIMER_CLOCK0, used to measure exact timing before answering
-    //   TIMER_CLOCK1, used to capture edges of the tag frames
+    //   Timer Counter 0, used to measure exact timing before answering
+    //   Timer Counter 1, used to capture edges of the tag frames
     AT91C_BASE_PMC->PMC_PCER |= (1 << AT91C_ID_TC0) | (1 << AT91C_ID_TC1);
 
     AT91C_BASE_PIOA->PIO_BSR = GPIO_SSC_FRAME;
@@ -880,12 +886,12 @@ void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontr
     AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
     AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
 
-    // TC0: Capture mode, default timer source = MCK/2 (TIMER_CLOCK1), no triggers
-    AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK;
+    // TC0: Capture mode, default timer source = MCK/32 (TIMER_CLOCK3), no triggers
+    AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK;
 
-    // TC1: Capture mode, default timer source = MCK/2 (TIMER_CLOCK1), TIOA is external trigger,
+    // TC1: Capture mode, default timer source = MCK/32 (TIMER_CLOCK3), TIOA is external trigger,
     // external trigger rising edge, load RA on rising edge of TIOA.
-    AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK
+    AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK
                              | AT91C_TC_ETRGEDG_RISING | AT91C_TC_ABETRG | AT91C_TC_LDRA_RISING;
 
     // Enable and reset counter
@@ -945,12 +951,12 @@ void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontr
             // Process the incoming frame (rx) and prepare the outgoing frame (tx)
             hitagS_handle_reader_command(rx, rxlen, tx, &txlen);
 
-            // Wait for HITAG_T_WAIT_1 carrier periods after the last reader bit,
+            // Wait for HITAG_T_WAIT_RESP carrier periods after the last reader bit,
             // not that since the clock counts since the rising edge, but T_Wait1 is
             // with respect to the falling edge, we need to wait actually (T_Wait1 - T_Low)
             // periods. The gap time T_Low varies (4..10). All timer values are in
             // terms of T0 units
-            while (AT91C_BASE_TC0->TC_CV < T0 * (HITAG_T_WAIT_1 - HITAG_T_LOW)) {};
+            while (AT91C_BASE_TC0->TC_CV < T0 * (HITAG_T_WAIT_RESP - HITAG_T_LOW)) {};
 
             // Send and store the tag answer (if there is any)
             if (txlen > 0) {
@@ -995,46 +1001,40 @@ static void hitagS_receive_frame(uint8_t *rx, size_t sizeofrx, size_t *rxlen, ui
     bool bSkip = true;
     *resptime = 0;
     uint32_t errorCount = 0;
-    // clk overflow but I failed moving TC0 & TC1 to
-    // slower clock AT91C_TC_CLKS_TIMER_DIV3_CLOCK
-    // so tracking overflow manually...
-    uint32_t overcount = 0;
-    uint32_t prevcv = 0;
     bool bStarted = false;
 
-    // Receive frame, watch for at most T0*EOF periods
-    while (AT91C_BASE_TC0->TC_CV + (overcount << 16) < (T0 * HITAG_T_PROG_MAX)) {
+    uint32_t ra_i = 0, h2 = 0, h3 = 0, h4 = 0;
+    uint8_t edges[160] = {0};
 
-        // detect and track counter overflows
-        uint32_t tmpcv = AT91C_BASE_TC0->TC_CV;
-        if (tmpcv < prevcv) {
-            overcount++;
-        }
-        prevcv = tmpcv;
+    // Dbprintf("TC0_CV:%i TC1_CV:%i TC1_RA:%i", AT91C_BASE_TC0->TC_CV, AT91C_BASE_TC1->TC_CV ,AT91C_BASE_TC1->TC_RA);
+
+    // Receive frame, watch for at most T0*HITAG_T_PROG_MAX periods
+    while (AT91C_BASE_TC0->TC_CV < (T0 * HITAG_T_PROG_MAX)) {
 
         // Check if falling edge in tag modulation is detected
         if (AT91C_BASE_TC1->TC_SR & AT91C_TC_LDRAS) {
 
             // Retrieve the new timing values
-            uint32_t ra = (AT91C_BASE_TC1->TC_RA + (overcount << 16)) / T0;
-
+            uint32_t ra = AT91C_BASE_TC1->TC_RA / T0;
+            edges[ra_i++] = ra;
             // Reset timer every frame, we have to capture the last edge for timing
             AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
 
-            prevcv = 0;
-            overcount = 0;
-
             if (ledcontrol) LED_B_ON();
 
             // Capture tag frame (manchester decoding using only falling edges)
 
             if (bStarted == false) {
 
-                if (ra >= HITAG_T_EOF) {
+                // Capture the T0 periods that have passed since last communication or field drop (reset)
+                *resptime = ra - HITAG_T_TAG_HALF_PERIOD;
+
+                if (ra >= HITAG_T_WAIT_RESP) {
                     bStarted = true;
-                    // Capture the T0 periods that have passed since last communication or field drop (reset)
+
                     // We always receive a 'one' first, which has the falling edge after a half period |-_|
-                    *resptime = ra - HITAG_T_TAG_HALF_PERIOD;
+                    rx[0] = 0x80;
+                    (*rxlen)++;
                 } else {
                     errorCount++;
                 }
@@ -1047,7 +1047,7 @@ static void hitagS_receive_frame(uint8_t *rx, size_t sizeofrx, size_t *rxlen, ui
 
                 rx[(*rxlen) / 8] |= 1 << (7 - ((*rxlen) % 8));
                 (*rxlen)++;
-
+                h4++;
             } else if (ra >= HITAG_T_TAG_CAPTURE_THREE_HALF) {
 
                 // Manchester coding example |_-|...|_-|-_| (0...01)
@@ -1062,13 +1062,13 @@ static void hitagS_receive_frame(uint8_t *rx, size_t sizeofrx, size_t *rxlen, ui
 
                 lastbit = !lastbit;
                 bSkip = !bSkip;
-
+                h3++;
             } else if (ra >= HITAG_T_TAG_CAPTURE_TWO_HALF) {
                 // Manchester coding example |_-|_-| (00) or |-_|-_| (11)
                 // bit is same as last bit
                 rx[(*rxlen) / 8] |= lastbit << (7 - ((*rxlen) % 8));
                 (*rxlen)++;
-
+                h2++;
             } else {
                 // Ignore weird value, is to small to mean anything
                 errorCount++;
@@ -1076,29 +1076,35 @@ static void hitagS_receive_frame(uint8_t *rx, size_t sizeofrx, size_t *rxlen, ui
         }
 
         // if we saw over 100 weird values break it probably isn't hitag...
-        if (errorCount > 100) {
+        if (errorCount > 100 || (*rxlen) / 8 >= sizeofrx) {
             break;
         }
 
         // We can break this loop if we received the last bit from a frame
-        if (AT91C_BASE_TC1->TC_CV > (T0 * HITAG_T_EOF)) {
+        // max periods between 2 falling edge
+        // RTF AC64 |--__|--__| (00) 64 * T0
+        // RTF MC32 |_-|-_|_-| (010) 48 * T0
+        if (AT91C_BASE_TC1->TC_CV > (T0 * 80)) {
             if ((*rxlen)) {
                 break;
             }
         }
     }
-    // Dbprintf("RX0 %i:%02X.. err:%i resptime:%i", *rxlen, rx[0], errorCount, *resptime);
+    if (g_dbglevel >= DBG_EXTENDED) {
+        Dbprintf("RX0 %i:%02X.. err:%i resptime:%i h2:%i h3:%i h4:%i edges:", *rxlen, rx[0], errorCount, *resptime, h2, h3, h4);
+        Dbhexdump(ra_i, edges, false);
+    }
 }
 
 static void sendReceiveHitagS(const uint8_t *tx, size_t txlen, uint8_t *rx, size_t sizeofrx, size_t *prxbits, int t_wait, bool ledcontrol, bool ac_seq) {
 
-    LogTraceBits(tx, txlen, HITAG_T_WAIT_2, HITAG_T_WAIT_2, true);
+    LogTraceBits(tx, txlen, HITAG_T_WAIT_SC, HITAG_T_WAIT_SC, true);
 
     // Send and store the reader command
     // Disable timer 1 with external trigger to avoid triggers during our own modulation
     AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
 
-    // Wait for HITAG_T_WAIT_2 carrier periods after the last tag bit before transmitting,
+    // Wait for HITAG_T_WAIT_SC carrier periods after the last tag bit before transmitting,
     // Since the clock counts since the last falling edge, a 'one' means that the
     // falling edge occurred halfway the period. with respect to this falling edge,
     // we need to wait (T_Wait2 + half_tag_period) when the last was a 'one'.
@@ -1125,19 +1131,20 @@ static void sendReceiveHitagS(const uint8_t *tx, size_t txlen, uint8_t *rx, size
             response_bit[i] = (rx[i / 8] >> (7 - (i % 8))) & 1;
         }
 
-        Dbprintf("htS: rxlen...... %zu", rxlen);
-        Dbprintf("htS: sizeofrx... %zu", sizeofrx);
+        if (g_dbglevel >= DBG_EXTENDED) {
+            Dbprintf("htS: rxlen...... %zu", rxlen);
+            Dbprintf("htS: sizeofrx... %zu", sizeofrx);
+            DbpString("htS: response_bit:");
+            Dbhexdump(rxlen, response_bit, false);
+        }
 
         memset(rx, 0x00, sizeofrx);
 
         if (ac_seq) {
 
-            DbpString("htS: AntiCollision Sequence ( ac seq )");
-            Dbhexdump(rxlen, response_bit, false);
-
             // Tag Response is AC encoded
             // We used UID Request Advanced,  meaning AC SEQ header is  111.
-            for (int i = 6; i < rxlen; i += 2) {
+            for (int i = 7; i < rxlen; i += 2) {
 
                 rx[k / 8] |= response_bit[i] << (7 - (k % 8));
 
@@ -1148,16 +1155,26 @@ static void sendReceiveHitagS(const uint8_t *tx, size_t txlen, uint8_t *rx, size
                 }
             }
 
-            DbpString("htS: ac sequence compress");
-            Dbhexdump(k / 8, rx, false);
+            // TODO: It's very confusing to reinterpreter the MC to AC; we should implement a more straightforward approach.
+            // add the lost bit zero, when AC64 last bit is zero
+            if (k % 8 == 7) {
+                k++;
+            }
+
+            if (g_dbglevel >= DBG_EXTENDED) {
+                DbpString("htS: ac sequence compress");
+                Dbhexdump(k / 8, rx, false);
+            }
 
         } else {
 
-            DbpString("htS: skipping 5 bit header");
+            if (g_dbglevel >= DBG_EXTENDED) {
+                DbpString("htS: skipping 6 bit header");
+            }
 
-            // ignore first 5 bits: SOF (actually 1 or 6 depending on response protocol)
+            // ignore first 6 bits: SOF (actually 1 or 6 depending on response protocol)
             // or rather a header.
-            for (size_t i = 5; i < rxlen; i++) {
+            for (size_t i = 6; i < rxlen; i++) {
 
                 rx[k / 8] |= response_bit[i] << (7 - (k % 8));
                 k++;
@@ -1166,8 +1183,6 @@ static void sendReceiveHitagS(const uint8_t *tx, size_t txlen, uint8_t *rx, size
                     break;
                 }
             }
-
-
         }
         LogTraceBits(rx, k, resptime, resptime, false);
     }
@@ -1201,6 +1216,8 @@ static int selectHitagS(const lf_hitag_data_t *packet, uint8_t *tx, size_t sizeo
 
     if (ledcontrol) LED_D_ON();
 
+    hitagS_init_clock();
+
     // Set fpga in edge detect with reader field, we can modulate as reader now
     FpgaWriteConfWord(FPGA_MAJOR_MODE_LF_EDGE_DETECT | FPGA_LF_EDGE_DETECT_READER_FIELD);
     FpgaSendCommand(FPGA_CMD_SET_DIVISOR, LF_DIVISOR_125); //125kHz
@@ -1213,15 +1230,15 @@ static int selectHitagS(const lf_hitag_data_t *packet, uint8_t *tx, size_t sizeo
     // Disable modulation at default, which means enable the field
     LOW(GPIO_SSC_DOUT);
 
-    hitagS_init_clock();
+    // Dbprintf("TC0_CV:%i TC1_CV:%i TC1_RA:%i", AT91C_BASE_TC0->TC_CV, AT91C_BASE_TC1->TC_CV, AT91C_BASE_TC1->TC_RA);
 
     // UID request standard   00110
     // UID request Advanced   1100x
     // UID request FAdvanced  11010
     size_t txlen = 0;
     size_t rxlen = 0;
-    uint8_t cmd = 0x18;  // 11000  UID Request Advanced
-    txlen = concatbits(tx, txlen, &cmd, 8 - 5, 5);
+    uint8_t cmd = HITAGS_UID_REQ_ADV;
+    txlen = concatbits(tx, txlen, &cmd, 0, 5);
     sendReceiveHitagS(tx, txlen, rx, sizeofrx, &rxlen, t_wait, ledcontrol, true);
 
     if (rxlen != 32) {
@@ -1237,13 +1254,13 @@ static int selectHitagS(const lf_hitag_data_t *packet, uint8_t *tx, size_t sizeo
 
     //select uid
     txlen = 0;
-    cmd = 0x00;
-    txlen = concatbits(tx, txlen, &cmd, 8 - 5, 5);
+    cmd = HITAGS_SELECT;
+    txlen = concatbits(tx, txlen, &cmd, 0, 5);
     txlen = concatbits(tx, txlen, rx, 0, 32);
     uint8_t crc = CRC8Hitag1Bits(tx, txlen);
     txlen = concatbits(tx, txlen, &crc, 0, 8);
 
-    sendReceiveHitagS(tx, txlen, rx, sizeofrx, &rxlen, t_wait, ledcontrol, false);
+    sendReceiveHitagS(tx, txlen, rx, sizeofrx, &rxlen, HITAG_T_WAIT_SC, ledcontrol, false);
 
     if (rxlen != 40) {
         Dbprintf("Select UID failed! %i", rxlen);
@@ -1343,12 +1360,15 @@ static int selectHitagS(const lf_hitag_data_t *packet, uint8_t *tx, size_t sizeo
                 tx[i] = ((NrAr >> (56 - (i * 8))) & 0xFF);
             }
 
+        } else if (packet->cmd == RHTSF_PLAIN || packet->cmd == WHTSF_PLAIN) {
+            Dbprintf("Error, " _YELLOW_("AUT=1") " This tag is configured in Authentication Mode");
+            return -1;
         } else {
-            Dbprintf("Error , unknown function: " _RED_("%d"), packet->cmd);
+            Dbprintf("Error, unknown function: " _RED_("%d"), packet->cmd);
             return -1;
         }
 
-        sendReceiveHitagS(tx, txlen, rx, sizeofrx, &rxlen, t_wait, ledcontrol, false);
+        sendReceiveHitagS(tx, txlen, rx, sizeofrx, &rxlen, HITAG_T_WAIT_SC, ledcontrol, false);
 
         if (rxlen != 40) {
             Dbprintf("Authenticate failed! " _RED_("%i"), rxlen);
@@ -1399,9 +1419,7 @@ void ReadHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
 
     uint8_t tx[HITAG_FRAME_LEN];
 
-    int t_wait = HITAG_T_WAIT_MAX;
-
-    if (selectHitagS(payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), t_wait, ledcontrol) == -1) {
+    if (selectHitagS(payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), HITAG_T_WAIT_FIRST, ledcontrol) == -1) {
 
         hitagS_stop_clock();
         set_tracing(false);
@@ -1418,14 +1436,14 @@ void ReadHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
 
         //send read request
         size_t txlen = 0;
-        uint8_t cmd = 0x0c;
-        txlen = concatbits(tx, txlen, &cmd, 8 - 4, 4);
+        uint8_t cmd = HITAGS_READ_PAGE;
+        txlen = concatbits(tx, txlen, &cmd, 0, 4);
         uint8_t addr = pageNum;
         txlen = concatbits(tx, txlen, &addr, 0, 8);
         uint8_t crc = CRC8Hitag1Bits(tx, txlen);
         txlen = concatbits(tx, txlen, &crc, 0, 8);
 
-        sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, t_wait, ledcontrol, false);
+        sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, HITAG_T_WAIT_SC, ledcontrol, false);
 
         if (rxlen == 0) {
             Dbprintf("Read page failed!");
@@ -1437,17 +1455,19 @@ void ReadHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
             tag.pages[pageNum][i] = rx[i];
         }
 
-        if (tag.auth && tag.LKP && pageNum == 1) {
-            Dbprintf("Page[%2d]: %02X %02X %02X %02X", pageNum, pwdh0,
-                     (tag.pages[pageNum][2]) & 0xff,
-                     (tag.pages[pageNum][1]) & 0xff,
-                     tag.pages[pageNum][0] & 0xff);
-        } else {
-            Dbprintf("Page[%2d]: %02X %02X %02X %02X", pageNum,
-                     (tag.pages[pageNum][3]) & 0xff,
-                     (tag.pages[pageNum][2]) & 0xff,
-                     (tag.pages[pageNum][1]) & 0xff,
-                     tag.pages[pageNum][0] & 0xff);
+        if (g_dbglevel >= DBG_EXTENDED) {
+            if (tag.auth && tag.LKP && pageNum == 1) {
+                Dbprintf("Page[%2d]: %02X %02X %02X %02X", pageNum, pwdh0,
+                         (tag.pages[pageNum][2]) & 0xff,
+                         (tag.pages[pageNum][1]) & 0xff,
+                         tag.pages[pageNum][0] & 0xff);
+            } else {
+                Dbprintf("Page[%2d]: %02X %02X %02X %02X", pageNum,
+                         (tag.pages[pageNum][3]) & 0xff,
+                         (tag.pages[pageNum][2]) & 0xff,
+                         (tag.pages[pageNum][1]) & 0xff,
+                         tag.pages[pageNum][0] & 0xff);
+            }
         }
 
         pageNum++;
@@ -1504,11 +1524,9 @@ void WritePageHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
     uint8_t tx[HITAG_FRAME_LEN];
     size_t txlen = 0;
 
-    int t_wait = HITAG_T_WAIT_MAX;
-
     int res = PM3_ESOFT;
 
-    if (selectHitagS(payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), t_wait, ledcontrol) == -1) {
+    if (selectHitagS(payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), HITAG_T_WAIT_FIRST, ledcontrol) == -1) {
         res = PM3_ERFTRANS;
         goto write_end;
     }
@@ -1523,8 +1541,8 @@ void WritePageHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
     //send write page request
     txlen = 0;
 
-    uint8_t cmd = 0x08;
-    txlen = concatbits(tx, txlen, &cmd, 8 - 4, 4);
+    uint8_t cmd = HITAGS_WRITE_PAGE;
+    txlen = concatbits(tx, txlen, &cmd, 0, 4);
 
     uint8_t addr = payload->page;
     txlen = concatbits(tx, txlen, &addr, 0, 8);
@@ -1532,43 +1550,39 @@ void WritePageHitagS(const lf_hitag_data_t *payload, bool ledcontrol) {
     uint8_t crc = CRC8Hitag1Bits(tx, txlen);
     txlen = concatbits(tx, txlen, &crc, 0, 8);
 
-    sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, t_wait, ledcontrol, false);
+    sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, HITAG_T_WAIT_SC, ledcontrol, false);
 
-    if ((rxlen != 2) || (rx[0] >> (8 - 2) != 0x1)) {
+    if ((rxlen != 2) || (rx[0] >> (8 - 2) != 0x01)) {
         Dbprintf("no write access on page " _YELLOW_("%d"), payload->page);
         res = PM3_ESOFT;
         goto write_end;
     }
 
-    //ACK received to write the page. send data
-    uint8_t data[4] = {0, 0, 0, 0};
-    switch (payload->cmd) {
-        case WHTSF_CHALLENGE:
-            data[0] = payload->data[3];
-            data[1] = payload->data[2];
-            data[2] = payload->data[1];
-            data[3] = payload->data[0];
-            break;
-        case WHTSF_KEY:
-            data[0] = payload->data[3];
-            data[1] = payload->data[2];
-            data[2] = payload->data[1];
-            data[3] = payload->data[0];
-            break;
-        default: {
-            res = PM3_EINVARG;
-            goto write_end;
-        }
-    }
+    // //ACK received to write the page. send data
+    // uint8_t data[4] = {0, 0, 0, 0};
+    // switch (payload->cmd) {
+    //     case WHTSF_PLAIN:
+    //     case WHTSF_CHALLENGE:
+    //     case WHTSF_KEY:
+    //         data[0] = payload->data[3];
+    //         data[1] = payload->data[2];
+    //         data[2] = payload->data[1];
+    //         data[3] = payload->data[0];
+    //         break;
+    //     default: {
+    //         res = PM3_EINVARG;
+    //         goto write_end;
+    //     }
+    // }
 
     txlen = 0;
-    txlen = concatbits(tx, txlen, data, 0, 32);
+    txlen = concatbits(tx, txlen, payload->data, 0, 32);
     crc = CRC8Hitag1Bits(tx, txlen);
     txlen = concatbits(tx, txlen, &crc, 0, 8);
 
-    sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, t_wait, ledcontrol, false);
+    sendReceiveHitagS(tx, txlen, rx, ARRAYLEN(rx), &rxlen, HITAG_T_WAIT_SC, ledcontrol, false);
 
-    if ((rxlen != 2) || (rx[0] >> (8 - 2) != 0x1)) {
+    if ((rxlen != 2) || (rx[0] >> (8 - 2) != 0x01)) {
         res = PM3_ESOFT; //  write failed
     } else {
         res = PM3_SUCCESS;
@@ -1600,7 +1614,6 @@ void Hitag_check_challenges(const uint8_t *data, uint32_t datalen, bool ledcontr
 
     uint8_t rx[HITAG_FRAME_LEN];
     uint8_t tx[HITAG_FRAME_LEN];
-    int t_wait = HITAG_T_WAIT_MAX;
 
     while ((BUTTON_PRESS() == false) && (data_available() == false)) {
         // Watchdog hit
@@ -1612,7 +1625,7 @@ void Hitag_check_challenges(const uint8_t *data, uint32_t datalen, bool ledcontr
 
         memcpy(payload.NrAr, data + dataoffset, 8);
 
-        int res = selectHitagS(&payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), t_wait, ledcontrol);
+        int res = selectHitagS(&payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), HITAG_T_WAIT_FIRST, ledcontrol);
         Dbprintf("Challenge %s: %02X %02X %02X %02X %02X %02X %02X %02X",
                  res == -1 ? "failed " : "success",
                  payload.NrAr[0], payload.NrAr[1],
@@ -1625,7 +1638,7 @@ void Hitag_check_challenges(const uint8_t *data, uint32_t datalen, bool ledcontr
             // Need to do a dummy UID select that will fail
             FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
             SpinDelay(2);
-            selectHitagS(&payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), t_wait, ledcontrol);
+            selectHitagS(&payload, tx, ARRAYLEN(tx), rx, ARRAYLEN(rx), HITAG_T_WAIT_FIRST, ledcontrol);
         }
 
         dataoffset += 8;
