@@ -676,7 +676,7 @@ int DesfireAPDU(uint8_t *cmd, size_t cmd_len, uint8_t *dataout) {
 
     ReaderTransmit(wCmd, wrappedLen, NULL);
 
-    len = ReaderReceive(resp, par);
+    len = ReaderReceive(resp, sizeof(resp), par);
     if (!len) {
         if (g_dbglevel >= DBG_EXTENDED) Dbprintf("fukked");
         return false; //DATA LINK ERROR
