@@ -304,7 +304,7 @@ static bool MifareSimInit(uint16_t flags, uint8_t *datain, uint16_t atqa, uint8_
         }
 
         // Correct uid size bits in ATQA
-        rATQA[0] = (rATQA[0] & 0x3f) | 0x00; // single size uid
+        rATQA[0] = (rATQA[0] & 0x3f); // single size uid
 
     } else if ((flags & FLAG_7B_UID_IN_DATA) == FLAG_7B_UID_IN_DATA) {
         memcpy(&rUIDBCC1[1], datain, 3);
@@ -579,7 +579,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
 
         FpgaEnableTracing();
         //Now, get data
-        int res = EmGetCmd(receivedCmd, &receivedCmd_len, receivedCmd_par);
+        int res = EmGetCmd(receivedCmd, sizeof(receivedCmd), &receivedCmd_len, receivedCmd_par);
 
         if (res == 2) { //Field is off!
             //FpgaDisableTracing();
@@ -763,7 +763,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                 }
 
                 // all commands must have a valid CRC
-                if (!CheckCrc14A(receivedCmd_dec, receivedCmd_len)) {
+                if (CheckCrc14A(receivedCmd_dec, receivedCmd_len) == false) {
                     EmSend4bit(encrypted_data ? mf_crypto1_encrypt4bit(pcs, CARD_NACK_NA) : CARD_NACK_NA);
                     FpgaDisableTracing();
 
@@ -1076,7 +1076,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                 }
 
                 // case MFEMUL_WORK => CMD RATS
-                if (receivedCmd_len == 4 && receivedCmd_dec[0] == ISO14443A_CMD_RATS && receivedCmd_dec[1] == 0x80) {
+                if (receivedCmd_len == 4 && receivedCmd_dec[0] == ISO14443A_CMD_RATS && (receivedCmd_dec[1] & 0xF0) <= 0x80 && (receivedCmd_dec[1] & 0x0F) <= 0x0e) {
                     if (rats && rats_len) {
                         if (encrypted_data) {
                             memcpy(response, rats, rats_len);
@@ -1364,7 +1364,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                          , ar_nr_resp[i].sector
                          , (ar_nr_resp[i].keytype == AUTHKEYA) ? "key A" : "key B"
                         );
-                Dbprintf("../tools/mfkey/mfkey32 %08x %08x %08x %08x %08x %08x",
+                Dbprintf("../tools/mfc/card_reader/mfkey32 %08x %08x %08x %08x %08x %08x",
                          ar_nr_resp[i].cuid,  //UID
                          ar_nr_resp[i].nonce, //NT
                          ar_nr_resp[i].nr,    //NR1
@@ -1383,7 +1383,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                      , ar_nr_resp[i].sector
                      , (ar_nr_resp[i].keytype == AUTHKEYB) ? "key A" : "key B"
                     );
-            Dbprintf("../tools/mfkey/mfkey32v2 %08x %08x %08x %08x %08x %08x %08x",
+            Dbprintf("../tools/mfc/card_reader/mfkey32v2 %08x %08x %08x %08x %08x %08x %08x",
                      ar_nr_resp[i].cuid,  //UID
                      ar_nr_resp[i].nonce, //NT
                      ar_nr_resp[i].nr,    //NR1
