@@ -1,5 +1,5 @@
 /*
-** $Id: llex.c,v 2.63 2013/03/16 21:10:18 roberto Exp $
+** $Id: llex.c,v 2.63.1.2 2013/08/30 15:49:41 roberto Exp roberto $
 ** Lexical Analyzer
 ** See Copyright Notice in lua.h
 */
@@ -131,6 +131,8 @@ TString *luaX_newstring(LexState *ls, const char *str, size_t l) {
            table has no metatable, so it does not need to invalidate cache */
         setbvalue(o, 1);  /* t[string] = true */
         luaC_checkGC(L);
+    } else {  /* string already present */
+        ts = rawtsvalue(keyfromval(o));  /* re-use value previously stored */
     }
     L->top--;  /* remove string from stack */
     return ts;
@@ -148,7 +150,7 @@ static void inclinenumber(LexState *ls) {
     if (currIsNewline(ls) && ls->current != old)
         next(ls);  /* skip `\n\r' or `\r\n' */
     if (++ls->linenumber >= MAX_INT)
-        luaX_syntaxerror(ls, "chunk has too many lines");
+        lexerror(ls, "chunk has too many lines", 0);
 }
 
 
