@@ -49,7 +49,7 @@ int luaV_tostring(lua_State *L, StkId obj) {
     else {
         char s[LUAI_MAXNUMBER2STR];
         lua_Number n = nvalue(obj);
-        int l = lua_number2str(s, n);
+        int l = lua_number2str(s, sizeof(s), n);
         setsvalue2s(L, obj, luaS_newlstr(L, s, l));
         return 1;
     }
