@@ -44,7 +44,10 @@ static int openAIDFile(json_t **root, bool verbose) {
         goto out;
     }
 
-    PrintAndLogEx(DEBUG, "Loaded file " _YELLOW_("%s") " " _GREEN_("%zu") " records ( " _GREEN_("ok") " )", path, json_array_size(*root));
+    PrintAndLogEx(DEBUG, "Loaded file " _YELLOW_("%s") " " _GREEN_("%zu") " records ( " _GREEN_("ok") " )"
+                  , path
+                  , json_array_size(*root)
+                 );
 out:
     free(path);
     return retval;
