@@ -1,9 +1,17 @@
 //-----------------------------------------------------------------------------
-// Copyright (C) 2021 Merlok
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
 //
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // CIPURSE crypto primitives
 //-----------------------------------------------------------------------------
@@ -86,7 +94,7 @@ static uint64_t computeNLM48(uint64_t x, uint64_t y) {
     return res;
 }
 
-static void computeNLM(uint8_t *res, uint8_t *x, uint8_t *y) {
+static void computeNLM(uint8_t *res, const uint8_t *x, const uint8_t *y) {
     uint64_t x64 = 0;
     uint64_t y64 = 0;
 
@@ -213,7 +221,7 @@ bool CipurseCCheckCT(CipurseContext_t *ctx, uint8_t *CT) {
     return (memcmp(CT, ctx->CT, CIPURSE_AES_KEY_LENGTH) == 0);
 }
 
-static uint16_t CipurseCComputeMICCRC(uint8_t *data, size_t len) {
+static uint16_t CipurseCComputeMICCRC(const uint8_t *data, size_t len) {
     uint16_t initCRC = 0x6363;
     for (size_t i = 0; i < len; i++) {
         uint8_t ch = data[i] ^ initCRC;
