@@ -95,6 +95,16 @@ int CmdHFSearch(const char *Cmd) {
         }
     }
 
+    PROMPT_CLEARLINE;
+    PrintAndLogEx(INPLACE, " Searching for Topaz tag...");
+    if (IfPm3Iso14443a()) {
+        if (readTopazUid(false, false) == PM3_SUCCESS) {
+            PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("Topaz tag") " found\n");
+            success[TOPAZ] = true;
+            res = PM3_SUCCESS;
+        }
+    }
+
     PROMPT_CLEARLINE;
     PrintAndLogEx(INPLACE, " Searching for LTO-CM tag...");
     if (IfPm3Iso14443a()) {
@@ -119,7 +129,8 @@ int CmdHFSearch(const char *Cmd) {
         }
     }
 
-    // ICT
+    /*
+        // ICT
     if (IfPm3Iso14443a()) {
         int sel_state = infoHF14A(false, false, false);
         if (sel_state > 0) {
@@ -131,6 +142,7 @@ int CmdHFSearch(const char *Cmd) {
                 infoHF14A4Applications(verbose);
         }
     }
+    */
 
     PROMPT_CLEARLINE;
     PrintAndLogEx(INPLACE, " Searching for LEGIC tag...");
@@ -142,16 +154,6 @@ int CmdHFSearch(const char *Cmd) {
         }
     }
 
-    PROMPT_CLEARLINE;
-    PrintAndLogEx(INPLACE, " Searching for Topaz tag...");
-    if (IfPm3Iso14443a()) {
-        if (readTopazUid(false, false) == PM3_SUCCESS) {
-            PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("Topaz tag") " found\n");
-            success[TOPAZ] = true;
-            res = PM3_SUCCESS;
-        }
-    }
-
     // texkom
     PROMPT_CLEARLINE;
     PrintAndLogEx(INPLACE, " Searching for TEXKOM tag...");
@@ -176,7 +178,7 @@ int CmdHFSearch(const char *Cmd) {
     PROMPT_CLEARLINE;
     PrintAndLogEx(INPLACE, " Searching for ISO14443-B tag...");
     if (IfPm3Iso14443b()) {
-        if (readHF14B(false, false) == PM3_SUCCESS) {
+        if (readHF14B(false, false, false) == PM3_SUCCESS) {
             PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("ISO 14443-B tag") " found\n");
             success[ISO_14443B] = true;
             res = PM3_SUCCESS;
@@ -518,7 +520,7 @@ int CmdHFSniff(const char *Cmd) {
     return PM3_SUCCESS;
 }
 
-int handle_hf_plot(void) {
+int handle_hf_plot(bool show_plot) {
 
     uint8_t buf[FPGA_TRACE_SIZE] = {0};
 
@@ -534,7 +536,9 @@ int handle_hf_plot(void) {
 
     g_GraphTraceLen = FPGA_TRACE_SIZE;
 
-    ShowGraphWindow();
+    if (show_plot) {
+        ShowGraphWindow();
+    }
 
     // remove signal offset
     CmdHpf("");
@@ -558,7 +562,7 @@ int CmdHFPlot(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, true);
     CLIParserFree(ctx);
 
-    return handle_hf_plot();
+    return handle_hf_plot(true);
 }
 
 static int CmdHFList(const char *Cmd) {
