@@ -286,7 +286,7 @@ static int CmdHF14AJookiEncode(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf jooki encode",
                   "Encode a Jooki token to base64 NDEF URI format",
-                  "hf jooki encode -t            --> selftest\n"
+                  "hf jooki encode --test        --> self tests\n"
                   "hf jooki encode -r --dragon   --> read uid from tag and use for encoding\n"
                   "hf jooki encode --uid 04010203040506 --dragon\n"
                   "hf jooki encode --uid 04010203040506 --tid 1 --fid 1"
@@ -296,7 +296,7 @@ static int CmdHF14AJookiEncode(const char *Cmd) {
         arg_param_begin,
         arg_str0("u", "uid",  "<hex>", "uid bytes"),
         arg_lit0("r", NULL, "read uid from tag instead"),
-        arg_lit0("t", NULL, "self test"),
+        arg_lit0(NULL, "test", "self test"),
         arg_lit0("v", "verbose", "verbose output"),
         arg_lit0(NULL, "dragon", "figurine type"),
         arg_lit0(NULL, "fox", "figurine type"),
@@ -442,8 +442,8 @@ static int CmdHF14AJookiDecode(const char *Cmd) {
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
-    int dlen = 16;
     uint8_t b64[JOOKI_B64_LEN] = {0x00};
+    int dlen = sizeof(b64) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     memset(b64, 0x0, sizeof(b64));
     CLIGetStrWithReturn(ctx, 1, b64, &dlen);
     bool verbose = arg_get_lit(ctx, 2);
@@ -471,8 +471,8 @@ static int CmdHF14AJookiSim(const char *Cmd) {
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
-    int dlen = 16;
     uint8_t b64[JOOKI_B64_LEN] = {0x00};
+    int dlen = sizeof(b64) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     memset(b64, 0x0, sizeof(b64));
     CLIGetStrWithReturn(ctx, 1, b64, &dlen);
     CLIParserFree(ctx);
@@ -611,8 +611,8 @@ static int CmdHF14AJookiClone(const char *Cmd) {
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
-    int blen = 16;
     uint8_t b64[JOOKI_B64_LEN] = {0x00};
+    int blen = sizeof(b64) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     memset(b64, 0x0, sizeof(b64));
     CLIGetStrWithReturn(ctx, 1, b64, &blen);
 
