@@ -30,6 +30,7 @@
 #include "crapto1/crapto1.h"
 #include "protocols.h"
 #include "cmdhficlass.h"
+#include "mifare/mifaredefault.h"  // mifare consts
 
 enum MifareAuthSeq {
     masNone,
@@ -382,7 +383,7 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
                     snprintf(exp, size, "WRITEBLOCK(" _MAGENTA_("%d") ")", cmd[1]);
                 else
                     // outside limits, useful for some tags...
-                    snprintf(exp, size, "WRITEBLOCK(" _MAGENTA_("%d") ") (?)", cmd[1]);
+                    snprintf(exp, size, "WRITEBLOCK(" _MAGENTA_("%d") ") (%s)", cmd[1], sprint_hex_inrow(cmd + 2, 4));
                 break;
             }
             case MIFARE_ULEV1_READ_CNT : {
@@ -430,6 +431,9 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
             default:
                 if ((cmd[0] & 0xF0) == 0xD0  && (cmdsize == 4 || cmdsize == 5)) {
                     snprintf(exp, size, "PPS - CID=%x", cmd[0] & 0x0F) ;
+                } else if ((cmd[0] & 0xF0) == 0x60  && (cmdsize == 4)) {
+                    MifareAuthState = masNt;
+                    snprintf(exp, size, "AUTH-%02X(" _MAGENTA_("%d") ")", cmd[0], cmd[1]);
                 } else {
                     return PM3_ESOFT;
                 }
@@ -2060,14 +2064,19 @@ void annotateMifare(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize,
     switch (MifareAuthState) {
         case masNt:
             if (cmdsize == 4 && isResponse) {
-                snprintf(exp, size, "AUTH: nt %s", (AuthData.first_auth) ? "" : "(enc)");
                 MifareAuthState = masNrAr;
                 if (AuthData.first_auth) {
                     AuthData.nt = bytes_to_num(cmd, 4);
                     AuthData.nt_enc_par = 0;
+                    if (validate_prng_nonce(AuthData.nt)) {
+                        snprintf(exp, size, "AUTH: nt (lfsr16 index %i)", nonce_distance(0, AuthData.nt));
+                    } else {
+                        snprintf(exp, size, "AUTH: nt");
+                    }
                 } else {
                     AuthData.nt_enc = bytes_to_num(cmd, 4);
                     AuthData.nt_enc_par = parity[0] & 0xF0;
+                    snprintf(exp, size, "AUTH: nt (enc)");
                 }
                 return;
             } else {
@@ -2227,7 +2236,7 @@ bool DecodeMifareData(uint8_t *cmd, uint8_t cmdsize, uint8_t *parity, bool isRes
                 mf_get_paritybinstr(sat, AuthData.at_enc, AuthData.at_enc_par);
 
                 PrintAndLogEx(NORMAL, "Nested authentication detected!");
-                PrintAndLogEx(NORMAL, "tools/mf_nonce_brute/mf_nonce_brute %x %x %s %x %x %s %x %s %s\n"
+                PrintAndLogEx(NORMAL, "tools/mfc/card_reader/mf_nonce_brute %x %x %s %x %x %s %x %s %s\n"
                               , AuthData.uid
                               , AuthData.nt_enc
                               , snt
