@@ -5,15 +5,16 @@
 #include "cmdhfxerox.h"
 
 #include "fileutils.h"
-
-#include "cmdparser.h"    // command_t
+#include "cmdtrace.h"
+#include "cmdparser.h"      // command_t
 #include "cliparser.h"
 #include "comms.h"
 #include "iso14b.h"
 #include "crc16.h"
+#include "commonutil.h"     // ARRAYLEN
 
 #define TIMEOUT 2000
-
+#define XEROX_BLOCK_SIZE    4
 
 #define c2l(c,l)    (l = ((unsigned long)(*((c)++))), \
              l |= ((unsigned long)(*((c)++))) << 8L, \
@@ -143,6 +144,9 @@ static const uint8_t lut[256] = {
 
 static const uint8_t var_list[] = {0x1c, 0x1e, 0x20, 0x26, 0x28, 0x2a, 0x2c, 0x2e};
 
+static uint8_t info_blocks[] = { 0x15, 0x16, 0x17, 0x18, 0x22 }; //Used for partnumber
+static const char *xerox_c_type[] = { "drum", "yellow", "magenta", "cyan", "black" };
+
 
 static int CmdHelp(const char *Cmd);
 void RC2_set_key(RC2_KEY *key, int len, const unsigned char *data, int bits);
@@ -381,44 +385,45 @@ static int switch_off_field(void) {
     return PM3_SUCCESS;
 }
 
-static int findXerox(iso14b_card_select_t *card, bool disconnect) {
+static int xerox_select_card(iso14b_card_select_t *card, bool disconnect) {
 
-    if (card == NULL)
+    if (card == NULL) {
         return PM3_EINVARG;
+    }
 
-    int8_t retry = 3;
+    int8_t retry = 2;
     while (retry--) {
 
         iso14b_raw_cmd_t packet = {
-            .flags = (ISO14B_CONNECT | ISO14B_SELECT_XRX | (disconnect ? ISO14B_DISCONNECT : 0)),
+            .flags = (ISO14B_CONNECT | ISO14B_SELECT_XRX),
             .timeout = 0,
             .rawlen = 0,
         };
+
+        if (disconnect) {
+            packet.flags |= ISO14B_DISCONNECT;
+        }
+
         clearCommandBuffer();
         SendCommandNG(CMD_HF_ISO14443B_COMMAND, (uint8_t *)&packet, sizeof(iso14b_raw_cmd_t));
         PacketResponseNG resp;
-        if (WaitForResponseTimeout(CMD_HF_ISO14443B_COMMAND, &resp, TIMEOUT)) {
+        if (WaitForResponseTimeout(CMD_HF_ISO14443B_COMMAND, &resp, TIMEOUT) == false) {
+            continue;
+        }
 
-            if (resp.oldarg[0] == 0) {
-                memcpy(card, (iso14b_card_select_t *)resp.data.asBytes, sizeof(iso14b_card_select_t));
-            }
-            return resp.oldarg[0];
+        if (resp.status == PM3_SUCCESS) {
+            memcpy(card, (iso14b_card_select_t *)resp.data.asBytes, sizeof(iso14b_card_select_t));
+            return resp.status;
         }
     } // retry
-
-//    switch_off_field();
-    PrintAndLogEx(FAILED, "command execution timeout");
     return PM3_ESOFT;
 }
 
-static uint8_t info_blocks[] = { 0x15, 0x16, 0x17, 0x18, 0x22 };
-static const char *c_type[] = { "drum", "yellow", "magenta", "cyan", "black" };
-
 static inline char dec_digit(uint8_t dig) {
     return (dig <= 9) ? dig + '0' : '?';
 }
 
-static void gen_pn(const uint8_t *data, char *pn) {
+static void xerox_generate_partno(const uint8_t *data, char *pn) {
     pn[0]  = dec_digit(data[0] >> 4);
     pn[1]  = dec_digit(data[0] & 0xF);
     pn[2]  = dec_digit(data[1] >> 4);
@@ -437,25 +442,155 @@ static void gen_pn(const uint8_t *data, char *pn) {
     pn[12] = 0;
 }
 
+static void xerox_print_hdr(void) {
+    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(INFO, "-------- " _CYAN_("tag memory") " ---------");
+    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(INFO, "block#   | data         | ascii");
+    PrintAndLogEx(INFO, "---------+--------------+----------");
+}
+
+static void xerox_print(uint8_t *data, uint16_t datalen, bool dense_output) {
+
+    uint16_t blockno = datalen / XEROX_BLOCK_SIZE;
+
+    bool in_repeated_block = false;
+
+
+    for (int i = 0; i < blockno; i++) {
+        // suppress repeating blocks, truncate as such that the first and last block with the same data is shown
+        // but the blocks in between are replaced with a single line of "......" if dense_output is enabled
+        uint8_t *blk = data + (i * XEROX_BLOCK_SIZE);
+        if (dense_output &&
+                (i > 3) &&
+                (i < (blockno - 1)) &&
+                (in_repeated_block == false) &&
+                (memcmp(blk, blk - XEROX_BLOCK_SIZE, XEROX_BLOCK_SIZE) == 0) &&
+                (memcmp(blk, blk + XEROX_BLOCK_SIZE, XEROX_BLOCK_SIZE) == 0) &&
+                (memcmp(blk, blk + (XEROX_BLOCK_SIZE * 2), XEROX_BLOCK_SIZE) == 0)
+           ) {
+            // we're in a user block that isn't the first user block nor last two user blocks,
+            // and the current block data is the same as the previous and next two block
+            in_repeated_block = true;
+            PrintAndLogEx(INFO, "  ......");
+        } else if (in_repeated_block &&
+                   (memcmp(blk, blk + XEROX_BLOCK_SIZE, XEROX_BLOCK_SIZE) || i == blockno)
+                  ) {
+            // in a repeating block, but the next block doesn't match anymore, or we're at the end block
+            in_repeated_block = false;
+        }
+
+        if (in_repeated_block == false) {
+            PrintAndLogEx(INFO,
+                          "%3d/0x%02X | %s | %s",
+                          i,
+                          i,
+                          sprint_hex(data + (i * XEROX_BLOCK_SIZE), XEROX_BLOCK_SIZE),
+                          sprint_ascii(data + (i * XEROX_BLOCK_SIZE), XEROX_BLOCK_SIZE)
+                         );
+        }
+    }
+}
+
+static void xerox_print_footer(void) {
+    PrintAndLogEx(INFO, "---------+--------------+----------");
+    PrintAndLogEx(NORMAL, "");
+}
+
+
+// structure and database for uid -> tagtype lookups
+typedef struct {
+    //General details
+    const char *partnumber;
+
+    //Toner related details
+    const char *color; // cyan, magenta, gold, silver, clear, white, fluo
+
+    const char *region; // DMO, WW, NA/ESG (NA - North America, MNA - Metred North America, DMO - Developing Markets, XE - Europe
+    const char *ms; // sold, metered
+
+    //Drum related details
+    const char *r; // Interchangeability with drumtray(s): color, black, both
+} xerox_part_t;
+
+// Additional data based on model DCP 550/560/570/C60/C70, PrimeLink C9065/C9070, WC 7965/7975 (Color)
+// https://gist.github.com/JeroenSteen/4b45886b8d87fa0530af9b0364e6b277
+// Todo: import data as actual JSON-file, and find a way to seperate toner/drums (with inheritance of consumable as type)
+static const xerox_part_t xerox_part_mappings[] = {
+    //Toner items
+    {"006R01532", "cyan", "DMO", "sold", ""},
+    {"006R01660", "cyan", "DMO", "sold", ""},
+    {"006R01739", "cyan", "DMO", "sold", ""},
+    {"006R01524", "cyan", "WW", "metered", ""},
+    {"006R01528", "cyan", "NA/ESG", "sold", ""},
+    {"006R01656", "cyan", "NA/ESG", "sold", ""},
+    {"006R01735", "cyan", "NA/ESG", "sold", ""},
+
+    {"006R01531", "magenta", "DMO", "sold", ""},
+    {"006R01661", "magenta", "DMO", "sold", ""},
+    {"006R01740", "magenta", "DMO", "sold", ""},
+    {"006R01523", "magenta", "WW", "metered", ""},
+    {"006R01527", "magenta", "NA/ESG", "sold", ""},
+    {"006R01657", "magenta", "NA/ESG", "sold", ""},
+    {"006R01736", "magenta", "NA/ESG", "sold", ""},
+
+    {"006R01530", "yellow", "DMO", "sold", ""},
+    {"006R01662", "yellow", "DMO", "sold", ""},
+    {"006R01741", "yellow", "DMO", "sold", ""},
+    {"006R01522", "yellow", "WW", "metered", ""},
+    {"006R01526", "yellow", "NA/ESG", "sold", ""},
+    {"006R01658", "yellow", "NA/ESG", "sold", ""},
+    {"006R01737", "yellow", "NA/ESG", "sold", ""},
+
+    {"006R01529", "black", "DMO", "sold", ""},
+    {"006R01659", "black", "DMO", "sold", ""},
+    {"006R01738", "black", "DMO", "sold", ""},
+    {"006R01521", "black", "WW", "metered", ""},
+    {"006R01525", "black", "NA/ESG", "sold", ""},
+    {"006R01655", "black", "NA/ESG", "sold", ""},
+    {"006R01734", "black", "NA/ESG", "sold", ""},
+
+    //Drum items
+    {"013R00663", "", "", "", "black"},
+    {"013R00664", "", "", "", "color"},
+
+    {"", "", "", "", ""} // must be the last entry
+};
+
+// get a product description based on the UID
+// returns description of the best match
+static const xerox_part_t *get_xerox_part_info(const char *pn) {
+    for (int i = 0; i < ARRAYLEN(xerox_part_mappings); i++) {
+        // Todo: make str_startswith, accept additional "Maximum number of characters to compare"
+        if (strncmp(pn, xerox_part_mappings[i].partnumber, strlen(pn) - 3) == 0) {
+            return &xerox_part_mappings[i];
+        }
+    }
+    //No match, return default
+    return &xerox_part_mappings[ARRAYLEN(xerox_part_mappings) - 1];
+}
+
 int read_xerox_uid(bool loop, bool verbose) {
 
     do {
         iso14b_card_select_t card;
-        int status = findXerox(&card, true);
+        int status = xerox_select_card(&card, true);
 
         if (loop) {
             if (status != PM3_SUCCESS) {
                 continue;
             }
-        } else {
+        }
 
-            if (status == PM3_SUCCESS) {
-                PrintAndLogEx(NORMAL, "");
-                PrintAndLogEx(SUCCESS, " UID    : %s", sprint_hex(card.uid, card.uidlen));
-                PrintAndLogEx(SUCCESS, " ATQB   : %s", sprint_hex(card.atqb, sizeof(card.atqb)));
-            } else {
-                return PM3_ESOFT;
+        if (status == PM3_SUCCESS) {
+            PrintAndLogEx(NORMAL, "");
+            PrintAndLogEx(SUCCESS, " UID..... %s", sprint_hex(card.uid, card.uidlen));
+            PrintAndLogEx(SUCCESS, " ATQB.... %s", sprint_hex(card.atqb, sizeof(card.atqb)));
+        } else {
+            if (verbose) {
+                PrintAndLogEx(WARNING, "no tag found");
             }
+            return PM3_ESOFT;
         }
 
     } while (loop && kbd_enter_pressed() == false);
@@ -463,11 +598,106 @@ int read_xerox_uid(bool loop, bool verbose) {
     return PM3_SUCCESS;
 }
 
+static void xerox_print_info(uint8_t *d) {
+
+    char pn[13];
+    xerox_generate_partno(d, pn);
+    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(INFO, "-------- " _CYAN_("tag info") " -----------");
+    PrintAndLogEx(SUCCESS, " PartNo....... " _YELLOW_("%s"), pn);
+    PrintAndLogEx(SUCCESS, " Date......... %02d.%02d.%02d", d[8], d[9], d[10]);
+    PrintAndLogEx(SUCCESS, " Serial....... %d", (d[14] << 16) | (d[13] << 8) | d[12]);
+    PrintAndLogEx(SUCCESS, " Type......... %s", (d[18] <= 4) ? xerox_c_type[d[18]] : "Unknown");
+
+    const xerox_part_t *item = get_xerox_part_info(pn);
+    if (strlen(item->partnumber) > 0) {
+
+        if (strcmp(xerox_c_type[d[18]], "drum") == 0) {
+
+            PrintAndLogEx(SUCCESS, " Consumable... drum");
+            PrintAndLogEx(SUCCESS, " Slots........ %s", item->r); // Interchangeability
+        } else {
+            PrintAndLogEx(SUCCESS, " Consumable... toner");
+            PrintAndLogEx(SUCCESS, " Region....... %s", item->region);
+            PrintAndLogEx(SUCCESS, " M/s.......... %s", item->ms);
+        }
+    }
+}
+
+
+// Assumes the card is already selected.
+static int read_xerox_block(iso14b_card_select_t *card, uint8_t blockno, uint8_t *out) {
+
+    if (card == NULL || out == NULL) {
+        return PM3_EINVARG;
+    }
+
+    uint8_t approx_len = (2 + card->uidlen + 1);
+    iso14b_raw_cmd_t *packet = (iso14b_raw_cmd_t *)calloc(1, sizeof(iso14b_raw_cmd_t) + approx_len);
+    if (packet == NULL) {
+        PrintAndLogEx(FAILED, "failed to allocate memory");
+        return PM3_EMALLOC;
+    }
+
+    // set up the read command
+    packet->flags = (ISO14B_APPEND_CRC | ISO14B_RAW);
+    packet->raw[packet->rawlen++] = 0x02;
+    packet->raw[packet->rawlen++] = (blockno < 12) ? ISO14443B_XEROX_READ_BLK : ISO14443B_XEROX_EXT_READ_BLK;
+
+    // uid
+    memcpy(packet->raw + packet->rawlen, card->uid, card->uidlen);
+    packet->rawlen += card->uidlen;
+
+    // block to read
+    packet->raw[packet->rawlen++] = blockno;
+
+
+    int res = PM3_ESOFT;
+    // read loop
+    for (int retry = 0; retry < 2; retry++) {
+        clearCommandBuffer();
+        SendCommandNG(CMD_HF_ISO14443B_COMMAND, (uint8_t *)packet, sizeof(iso14b_raw_cmd_t) + packet->rawlen);
+        PacketResponseNG resp;
+        if (WaitForResponseTimeout(CMD_HF_ISO14443B_COMMAND, &resp, 2000) == false) {
+            continue;
+        }
+
+        if (resp.status != PM3_SUCCESS) {
+            PrintAndLogEx(FAILED, "trying one more time");
+            continue;
+        }
+
+        // sanity checks
+        if (resp.length < 7) {
+            PrintAndLogEx(FAILED, "trying one more time, wrong length");
+            continue;
+        }
+
+        uint8_t *d = resp.data.asBytes;
+        if (check_crc(CRC_14443_B, d, 7) == false) {
+            PrintAndLogEx(FAILED, "trying one more time, CRC ( " _RED_("fail") " )");
+            continue;
+        }
+
+        if (d[0] != 0x02) {
+            PrintAndLogEx(FAILED, "Tag returned Error %x %x", d[0], d[1]);
+            res = PM3_ERFTRANS;
+            break;
+        }
+
+        memcpy(out, d + 1, XEROX_BLOCK_SIZE);
+        res = PM3_SUCCESS;
+        break;
+    }
+    return res;
+}
+
 static int CmdHFXeroxReader(const char *Cmd) {
 
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf xerox reader",
-                  "Act as a 14443B reader to identify a tag",
+                  "Act as a 14443B reader to identify a Fuji Xerox based tag\n"
+                  "ISO/IEC 14443 type B based communications",
                   "hf xerox reader\n"
                   "hf xerox reader -@ \n"
                  );
@@ -493,7 +723,8 @@ static int CmdHFXeroxReader(const char *Cmd) {
 static int CmdHFXeroxInfo(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf xerox info",
-                  "Tag information for ISO/IEC 14443 type B / XEROX based tags",
+                  "Tag information for Fuji Xerox based tags\n"
+                  "ISO/IEC 14443 type B based communications",
                   "hf xerox info"
                  );
 
@@ -507,96 +738,40 @@ static int CmdHFXeroxInfo(const char *Cmd) {
     CLIParserFree(ctx);
 
     iso14b_card_select_t card;
-    int status = findXerox(&card, false);
+    int status = xerox_select_card(&card, false);
     if (status != PM3_SUCCESS) {
-        switch_off_field();
         if (verbose) {
             PrintAndLogEx(FAILED, "Fuji/Xerox tag select failed");
         }
-        return PM3_ERFTRANS;
+        return status;
     }
 
     PrintAndLogEx(NORMAL, "");
-    PrintAndLogEx(SUCCESS, " UID    : %s", sprint_hex(card.uid, card.uidlen));
-    PrintAndLogEx(SUCCESS, " ATQB   : %s", sprint_hex(card.atqb, sizeof(card.atqb)));
+    PrintAndLogEx(SUCCESS, " UID..... %s", sprint_hex(card.uid, card.uidlen));
+    PrintAndLogEx(SUCCESS, " ATQB.... %s", sprint_hex(card.atqb, sizeof(card.atqb)));
 
-    iso14b_raw_cmd_t *packet = (iso14b_raw_cmd_t *)calloc(1, sizeof(iso14b_raw_cmd_t) + 11);
-    if (packet == NULL) {
-        PrintAndLogEx(FAILED, "failed to allocate memory");
-        return PM3_EMALLOC;
-    }
+    uint8_t data[ARRAYLEN(info_blocks) * XEROX_BLOCK_SIZE] = {0};
 
-    int blocknum = 0;
-    uint8_t data[sizeof(info_blocks) * 4] = {0};
+    // special blocks to read.
+    for (int i = 0; i < ARRAYLEN(info_blocks); i++) {
 
-    // set up the read command
-    packet->flags = (ISO14B_APPEND_CRC | ISO14B_RAW);
-    packet->rawlen = 11;
-    packet->raw[0] = 0x02;
-    packet->raw[1] = 0x20;          // set command: read mem
-    memcpy(packet->raw + 2, card.uid, 8);       // store uid
-
-    for (int retry = 0; (retry < 5 && blocknum < sizeof(info_blocks)); retry++) {
-
-        packet->raw[10] = info_blocks[blocknum];
-
-        PacketResponseNG resp;
-        clearCommandBuffer();
-        SendCommandNG(CMD_HF_ISO14443B_COMMAND, (uint8_t *)packet, sizeof(iso14b_raw_cmd_t) + packet->rawlen);
-        if (WaitForResponseTimeout(CMD_HF_ISO14443B_COMMAND, &resp, 2000)) {
-            /*
-            PrintAndLogEx(INFO, "%X %X %X %X %X %I64X %I64X %I64X %X %X %X %c",
-            resp.cmd, resp.length, resp.magic, resp.status, resp.crc, resp.oldarg[0], resp.oldarg[1], resp.oldarg[2],
-            resp.data.asBytes[0], resp.data.asBytes[1], resp.data.asBytes[2], resp.ng ? 't' : 'f');
-            */
-
-            // 14b raw command send data_len instead of status
-            if (/*resp.status != 0 ||*/ resp.length < 7) {
-                PrintAndLogEx(FAILED, "retrying one more time");
-                continue;
-            }
-
-            uint8_t *recv = resp.data.asBytes;
-
-            if (check_crc(CRC_14443_B, recv, 7) == false) {
-                PrintAndLogEx(FAILED, "crc fail, retrying one more time");
-                continue;
-            }
-
-            if (recv[0] != 2) {
-                PrintAndLogEx(FAILED, "Tag returned Error %x %x", recv[0], recv[1]);
-                break;
-            }
-
-            memcpy(data + (blocknum * 4), resp.data.asBytes + 1, 4);
-
-            retry = 0;
-            blocknum++;
+        status = read_xerox_block(&card, info_blocks[i], data + (i * XEROX_BLOCK_SIZE));
+        if (status != PM3_SUCCESS) {
+            PrintAndLogEx(FAILED, "Fuji/Xerox tag read failed");
+            break;
         }
     }
 
     switch_off_field();
-    free(packet);
-
-    if (blocknum != sizeof(info_blocks)) {
-        PrintAndLogEx(FAILED, "Fuji/Xerox tag read failed");
-        return PM3_ERFTRANS;
-    }
-
-    char pn[13];
-    gen_pn(data, pn);
-    PrintAndLogEx(SUCCESS, " PartNo : %s", pn);
-    PrintAndLogEx(SUCCESS, " Date   : %02d.%02d.%02d", data[8], data[9], data[10]);
-    PrintAndLogEx(SUCCESS, " Serial : %d", (data[14] << 16) | (data[13] << 8) | data[12]);
-    PrintAndLogEx(SUCCESS, " Type   : %s", (data[18] <= 4) ? c_type[data[18]] : "Unknown");
-
-    return PM3_SUCCESS;
+    xerox_print_info(data);
+    return status;
 }
 
 static int CmdHFXeroxDump(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf xerox dump",
-                  "Dump all memory from a Fuji/Xerox tag",
+                  "Dump all memory from a Fuji/Xerox tag\n"
+                  "ISO/IEC 14443 type B based communications",
                   "hf xerox dump\n"
                  );
 
@@ -604,6 +779,9 @@ static int CmdHFXeroxDump(const char *Cmd) {
         arg_param_begin,
         arg_str0("f", "file", "<fn>", "filename to save dump to"),
         arg_lit0("d", "decrypt", "decrypt secret blocks"),
+        arg_lit0(NULL, "ns", "no save to file"),
+        arg_lit0("v", "verbose", "verbose output"),
+        arg_lit0("z", "dense", "dense dump output style"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -612,85 +790,44 @@ static int CmdHFXeroxDump(const char *Cmd) {
     char filename[FILE_PATH_SIZE] = {0};
     CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
     bool decrypt = arg_get_lit(ctx, 2);
+    bool nosave = arg_get_lit(ctx, 3);
+    bool verbose = arg_get_lit(ctx, 4);
+    bool dense_output = (g_session.dense_output || arg_get_lit(ctx, 3));
     CLIParserFree(ctx);
 
-    iso14b_raw_cmd_t *packet = (iso14b_raw_cmd_t *)calloc(1, sizeof(iso14b_raw_cmd_t) + 11);
-    if (packet == NULL) {
-        PrintAndLogEx(FAILED, "failed to allocate memory");
-        return PM3_EMALLOC;
-    }
-
     iso14b_card_select_t card;
-    int status = findXerox(&card, false);   // remain RF on
+    int status = xerox_select_card(&card, false);
     if (status != PM3_SUCCESS) {
-        free(packet);
-        switch_off_field();
-        return PM3_ERFTRANS;
+        if (verbose) {
+            PrintAndLogEx(FAILED, "Fuji/Xerox tag select failed");
+        }
+        return status;
     }
 
     PrintAndLogEx(INFO, "Reading memory from tag UID " _GREEN_("%s"), sprint_hex(card.uid, card.uidlen));
+    PrintAndLogEx(INFO, "" NOLF);
 
-    int blocknum = 1;           // block 0 all zeros
-    uint8_t data[256 * 4] = {0};
+    uint8_t data[256 * XEROX_BLOCK_SIZE] = {0};
 
-    // set up the read command
-    packet->flags = (ISO14B_APPEND_CRC | ISO14B_RAW);
-    packet->rawlen = 11;
-    packet->raw[0] = 0x02;
-    memcpy(packet->raw + 2, card.uid, 8);       // store uid
+    uint16_t blockno = 0;
+    for (; blockno < 0x100; blockno++) {
 
-    PrintAndLogEx(INFO, "." NOLF);
-
-    for (int retry = 0; (retry < 5 && blocknum < 0x100); retry++) {
-
-        packet->raw[1]  = (blocknum < 12) ? 0x30 : 0x20;    // set command: read ext mem or read mem
-        packet->raw[10] = blocknum & 0xFF;
-
-        PacketResponseNG resp;
-        clearCommandBuffer();
-        SendCommandNG(CMD_HF_ISO14443B_COMMAND, (uint8_t *)packet, sizeof(iso14b_raw_cmd_t) + packet->rawlen);
-        if (WaitForResponseTimeout(CMD_HF_ISO14443B_COMMAND, &resp, 2000)) {
-            /*
-            PrintAndLogEx(INFO, "%X %X %X %X %X %I64X %I64X %I64X %X %X %X %c",
-            resp.cmd, resp.length, resp.magic, resp.status, resp.crc, resp.oldarg[0], resp.oldarg[1], resp.oldarg[2],
-            resp.data.asBytes[0], resp.data.asBytes[1], resp.data.asBytes[2], resp.ng ? 't' : 'f');
-            */
-            if (/*resp.status != 0 ||*/ resp.length < 7) {  // 14b raw command send data_len instead of status
-                PrintAndLogEx(FAILED, "retrying one more time");
-                continue;
-            }
-
-            uint8_t *recv = resp.data.asBytes;
-
-            if (check_crc(CRC_14443_B, recv, 7) == false) {
-                PrintAndLogEx(FAILED, "crc fail, retrying one more time");
-                continue;
-            }
-
-            if (recv[0] != 2) {
-                PrintAndLogEx(NORMAL, "");
-                PrintAndLogEx(FAILED, "Tag returned Error %x %x", recv[0], recv[1]);
-                break;
-            }
-
-            memcpy(data + (blocknum * 4), resp.data.asBytes + 1, 4);
-
-            retry = 0;
-            blocknum++;
-
-            PrintAndLogEx(NORMAL, "." NOLF);
-            fflush(stdout);
+        int res = read_xerox_block(&card, blockno, data + (blockno * XEROX_BLOCK_SIZE));
+        if (res != PM3_SUCCESS) {
+            PrintAndLogEx(FAILED, "Fuji/Xerox tag read failed");
+            break;
         }
+
+        PrintAndLogEx(INPLACE, "blk %3d", blockno);
     }
 
     switch_off_field();
 
-    free(packet);
-
     PrintAndLogEx(NORMAL, "");
 
-    if (blocknum != 0x100)
-        PrintAndLogEx(FAILED, "dump failed at block %d", blocknum);
+    if (blockno != 0x100) {
+        PrintAndLogEx(FAILED, "dump failed at block " _RED_("%d"), blockno);
+    }
 
     if (decrypt) {
         PrintAndLogEx(INFO, "Decrypting secret blocks...");
@@ -702,9 +839,9 @@ static int CmdHFXeroxDump(const char *Cmd) {
         k1[1] = data[5];
         k1[2] = data[6];
         k1[3] = data[7];
-        k1[4] = data[0x18 * 4 + 0];
-        k1[5] = data[0x18 * 4 + 1];
-        k1[6] = data[0x22 * 4 + 0];
+        k1[4] = data[(0x18 * XEROX_BLOCK_SIZE) + 0];
+        k1[5] = data[(0x18 * XEROX_BLOCK_SIZE) + 1];
+        k1[6] = data[(0x22 * XEROX_BLOCK_SIZE) + 0];
         k1[7] = 0;
 
         RC2_set_key(&exp_key, 8, k1, 64);
@@ -718,8 +855,8 @@ static int CmdHFXeroxDump(const char *Cmd) {
 
         memcpy(k1, k2, sizeof(k1));
 
-        k1[2] = k2[3] ^ data[0x22 * 4 + 0];
-        k1[3] = k2[4] ^ data[0x22 * 4 + 1]; // first_key[7];
+        k1[2] = k2[3] ^ data[(0x22 * XEROX_BLOCK_SIZE) + 0];
+        k1[3] = k2[4] ^ data[(0x22 * XEROX_BLOCK_SIZE) + 1]; // first_key[7];
         k1[5] = k2[1] ^ 0x01;       // 01 = crypto method? rfid[23][2]
 
         RC2_set_key(&exp_key, 8, k1, 64);
@@ -728,7 +865,7 @@ static int CmdHFXeroxDump(const char *Cmd) {
 
             uint8_t dadr = var_list[n];
 
-            if (dadr + 1 >= blocknum) {
+            if (dadr + 1 >= blockno) {
                 PrintAndLogEx(INFO, "secret block %02X skipped.", dadr);
                 continue;
             }
@@ -736,56 +873,152 @@ static int CmdHFXeroxDump(const char *Cmd) {
             memset(iv, 0, sizeof(iv));
             iv[0] = dadr;
 
-            RC2_cbc_encrypt(&data[dadr * 4], decr, 8, &exp_key, iv, RC2_DECRYPT);
+            RC2_cbc_encrypt(&data[dadr * XEROX_BLOCK_SIZE], decr, 8, &exp_key, iv, RC2_DECRYPT);
 
-            memcpy(&data[dadr * 4], decr, 8);
+            memcpy(&data[dadr * XEROX_BLOCK_SIZE], decr, 8);
 
             int b;
             uint16_t cs, csd;
 
             // calc checksum
-            for (b = 0, cs = 0; b < sizeof(decr) - 2; b += 2)   cs += decr[b] | (decr[b + 1] << 8);
+            for (b = 0, cs = 0; b < sizeof(decr) - 2; b += 2) {
+                cs += decr[b] | (decr[b + 1] << 8);
+            }
+
             cs = ~cs;
             csd = (decr[7] << 8) | decr[6];
 
             if (cs != csd) {
-                PrintAndLogEx(FAILED, "secret block %02X checksum failed.", dadr);
+                PrintAndLogEx(FAILED, "Secret block %02X checksum " _RED_("failed"), dadr);
             }
         }
     }
 
-    PrintAndLogEx(INFO, "block#   | data         | ascii");
-    PrintAndLogEx(INFO, "---------+--------------+----------");
+    xerox_print_hdr();
+    xerox_print(data, blockno * XEROX_BLOCK_SIZE, dense_output);
+    xerox_print_footer();
 
-    for (int i = 0; i < blocknum; i++) {
-        PrintAndLogEx(INFO,
-                      "%3d/0x%02X | %s | %s",
-                      i,
-                      i,
-                      sprint_hex(data + (i * 4), 4),
-                      sprint_ascii(data + (i * 4), 4)
-                     );
+    if (nosave) {
+        PrintAndLogEx(INFO, "Called with no save option");
+        PrintAndLogEx(NORMAL, "");
+        return PM3_SUCCESS;
     }
-    PrintAndLogEx(INFO, "---------+--------------+----------");
-    PrintAndLogEx(NORMAL, "");
 
     if (0 == filename[0]) { // generate filename from uid
         char *fptr = filename;
         PrintAndLogEx(INFO, "Using UID as filename");
         fptr += snprintf(fptr, sizeof(filename), "hf-xerox-");
-        FillFileNameByUID(fptr, SwapEndian64(card.uid, card.uidlen, 8), decrypt ? "-dump-dec" : "-dump", card.uidlen);
+        FillFileNameByUID(fptr
+                          , SwapEndian64(card.uid, card.uidlen, 8)
+                          , (decrypt) ? "-dump-dec" : "-dump"
+                          , card.uidlen
+                         );
     }
 
-    pm3_save_dump(filename, data, blocknum * 4, jsf14b_v2);
+    pm3_save_dump(filename, data, blockno * XEROX_BLOCK_SIZE, jsf14b_v2);
     return PM3_SUCCESS;
 }
 
+static int CmdHFXeroxView(const char *Cmd) {
+
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hf xerox view",
+                  "Print a Fuji/Xerox dump file (bin/eml/json)\n"
+                  "note:\n"
+                  "  - command expects the filename to contain a UID\n"
+                  "    which is needed to determine card memory type",
+                  "hf xerox view -f hf-xerox-0102030405060708-dump.bin"
+                 );
+    void *argtable[] = {
+        arg_param_begin,
+        arg_str1("f", "file", "<fn>", "Specify a filename for dump file"),
+        arg_lit0("v", "verbose", "verbose output"),
+        arg_lit0("z", "dense", "dense dump output style"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, false);
+    int fnlen = 0;
+    char filename[FILE_PATH_SIZE];
+    CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
+    bool verbose = arg_get_lit(ctx, 2);
+    bool dense_output = (g_session.dense_output || arg_get_lit(ctx, 3));
+    CLIParserFree(ctx);
+
+    // read dump file
+    uint8_t *dump = NULL;
+    size_t bytes_read = (XEROX_BLOCK_SIZE * 0x100);
+    int res = pm3_load_dump(filename, (void **)&dump, &bytes_read, (XEROX_BLOCK_SIZE * 0x100));
+    if (res != PM3_SUCCESS) {
+        return res;
+    }
+
+    uint16_t blockno = bytes_read / XEROX_BLOCK_SIZE;
+
+    if (verbose) {
+        PrintAndLogEx(INFO, "File size %zu bytes, file blocks %d (0x%x)", bytes_read, blockno, blockno);
+    }
+
+    uint8_t tmp[ARRAYLEN(info_blocks) * XEROX_BLOCK_SIZE] = {0};
+    for (uint8_t i = 0; i < ARRAYLEN(info_blocks); i++) {
+        memcpy(tmp + (i * XEROX_BLOCK_SIZE), dump + (info_blocks[i] * XEROX_BLOCK_SIZE), XEROX_BLOCK_SIZE);
+    }
+
+    xerox_print_info(tmp);
+    xerox_print_hdr();
+    xerox_print(dump, bytes_read, dense_output);
+    xerox_print_footer();
+
+    free(dump);
+    return PM3_SUCCESS;
+}
+
+static int CmdHFXeroxList(const char *Cmd) {
+    return CmdTraceListAlias(Cmd, "hf 14b", "14b -c");
+}
+
+static int CmdHFXeroxRdBl(const char *Cmd) {
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hf xerox rdbl",
+                  "Read a Fuji/Xerox tag block\n",
+                  "hf xerox rdbl -b 1"
+                 );
+    void *argtable[] = {
+        arg_param_begin,
+        arg_int1("b", "blk", "<dec>", "page number (0-255)"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, false);
+    int blockno = arg_get_int_def(ctx, 1, 0);
+    CLIParserFree(ctx);
+
+    iso14b_card_select_t card;
+    int status = xerox_select_card(&card, false);
+    if (status != PM3_SUCCESS) {
+        switch_off_field();
+        return status;
+    }
+
+    uint8_t block[XEROX_BLOCK_SIZE] = {0};
+    status = read_xerox_block(&card, (uint8_t)blockno, block);
+    switch_off_field();
+
+    if (status == PM3_SUCCESS) {
+        PrintAndLogEx(SUCCESS, "%i | %s", blockno, sprint_hex_ascii(block, sizeof(block)));
+    } else {
+        PrintAndLogEx(FAILED, "Fuji/Xerox tag read failed");
+    }
+    return status;
+}
+
 static command_t CommandTable[] = {
-    {"help",    CmdHelp,           AlwaysAvailable, "This help"},
-    {"info",    CmdHFXeroxInfo,    IfPm3Iso14443b,  "Short info on Fuji/Xerox tag"},
-    {"reader",  CmdHFXeroxReader,  IfPm3Iso14443b,  "Act like a Fuji/Xerox reader"},
-    {"dump",    CmdHFXeroxDump,    IfPm3Iso14443b,  "Read all memory pages of an Fuji/Xerox tag, save to file"},
-//    {"rdbl",    CmdHFXeroxRdBl,  IfPm3Iso14443b,  "Read Fuji/Xerox block"},
+    {"help",      CmdHelp,           AlwaysAvailable, "This help"},
+    {"list",      CmdHFXeroxList,    AlwaysAvailable, "List ISO-14443B history"},
+    {"--------",  CmdHelp,           AlwaysAvailable, "----------------------- " _CYAN_("General") " -----------------------"},
+    {"info",      CmdHFXeroxInfo,    IfPm3Iso14443b,  "Short info on Fuji/Xerox tag"},
+    {"dump",      CmdHFXeroxDump,    IfPm3Iso14443b,  "Read all memory pages of an Fuji/Xerox tag, save to file"},
+    {"reader",    CmdHFXeroxReader,  IfPm3Iso14443b,  "Act like a Fuji/Xerox reader"},
+    {"view",      CmdHFXeroxView,    AlwaysAvailable, "Display content from tag dump file"},
+    {"rdbl",      CmdHFXeroxRdBl,    IfPm3Iso14443b,  "Read Fuji/Xerox block"},
 //    {"wrbl",    CmdHFXeroxWrBl,  IfPm3Iso14443b,  "Write Fuji/Xerox block"},
     {NULL, NULL, NULL, NULL}
 };
