@@ -1358,8 +1358,8 @@ static int CmdConnect(const char *Cmd) {
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
 
-    int p_len = FILE_PATH_SIZE;
     char port[FILE_PATH_SIZE] = {0};
+    int p_len = sizeof(port) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated;
     CLIGetStrWithReturn(ctx, 1, (uint8_t *)port, &p_len);
     uint32_t baudrate = arg_get_u32_def(ctx, 2, USART_BAUD_RATE);
     CLIParserFree(ctx);
@@ -1432,7 +1432,7 @@ static int CmdBootloader(const char *Cmd) {
 }
 
 int set_fpga_mode(uint8_t mode) {
-    if (mode < FPGA_BITSTREAM_LF || mode > FPGA_BITSTREAM_HF_15) {
+    if (mode < FPGA_BITSTREAM_MIN || mode > FPGA_BITSTREAM_MAX) {
         return PM3_EINVARG;
     }
     uint8_t d[] = {mode};
@@ -1468,12 +1468,12 @@ static command_t CommandTable[] = {
     {"ping",          CmdPing,         IfPm3Present,     "Test if the Proxmark3 is responsive"},
     {"readmem",       CmdReadmem,      IfPm3Present,     "Read from MCU flash"},
     {"reset",         CmdReset,        IfPm3Present,     "Reset the device"},
-    {"setlfdivisor",  CmdSetDivisor,   IfPm3Present,     "Drive LF antenna at 12MHz / (divisor + 1)"},
-    {"sethfthresh",   CmdSetHFThreshold, IfPm3Present,   "Set thresholds in HF/14a mode"},
+    {"setlfdivisor",  CmdSetDivisor,   IfPm3Lf,          "Drive LF antenna at 12MHz / (divisor + 1)"},
+    {"sethfthresh",   CmdSetHFThreshold, IfPm3Iso14443a, "Set thresholds in HF/14a mode"},
     {"setmux",        CmdSetMux,       IfPm3Present,     "Set the ADC mux to a specific value"},
     {"standalone",    CmdStandalone,   IfPm3Present,     "Start installed standalone mode on device"},
     {"tia",           CmdTia,          IfPm3Present,     "Trigger a Timing Interval Acquisition to re-adjust the RealTimeCounter divider"},
-    {"tune",          CmdTune,         IfPm3Present,     "Measure tuning of device antenna"},
+    {"tune",          CmdTune,         IfPm3Lf,          "Measure tuning of device antenna"},
     {NULL, NULL, NULL, NULL}
 };
 
