@@ -34,7 +34,7 @@
 #include "cmdhw.h"          // for setting FPGA image
 #include "cmdlfawid.h"      // for awid menu
 #include "cmdlfem.h"        // for em menu
-#include "cmdlfem410x.h"      // for em4x menu
+#include "cmdlfem410x.h"    // for em4x menu
 #include "cmdlfem4x05.h"    // for em4x05 / 4x69
 #include "cmdlfem4x50.h"    // for em4x50
 #include "cmdlfem4x70.h"    // for em4x70
@@ -67,6 +67,7 @@
 #include "cmdlfzx8211.h"    // for ZX8211 menu
 #include "crc.h"
 #include "pm3_cmd.h"        // for LF_CMDREAD_MAX_EXTRA_SYMBOLS
+#include "fpga.h"           // for set_fpga_mode
 
 static int CmdHelp(const char *Cmd);
 
@@ -246,10 +247,11 @@ int CmdLFCommandRead(const char *Cmd) {
     CLIParserInit(&ctx, "lf cmdread",
                   "Modulate LF reader field to send command before read. All periods in microseconds.\n"
                   " - use " _YELLOW_("`lf config`") _CYAN_(" to set parameters"),
-                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W00110                           --> probing for Hitag1/S\n"
-                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W11000                           --> probing for Hitag2\n"
-                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W11000 -s 2000 -@                --> probing for Hitag2, oscilloscope style\n"
-                  "lf cmdread -d 48 -z 112 -o 176 -e W3000 -e S240 -e E336 -c W0S00000010000E  --> probing for Hitag (us)\n"
+                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W00110                           --> probing for Hitag 1/S\n"
+                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W11000                           --> probing for Hitag 2/S\n"
+                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W11010                           --> probing for Hitag S\n"
+                  "lf cmdread -d 50 -z 116 -o 166 -e W3000 -c W11000 -s 2000 -@                --> probing for Hitag 2/S, oscilloscope style\n"
+                  "lf cmdread -d 48 -z 112 -o 176 -e W3000 -e S240 -e E336 -c W0S00000010000E  --> probing for Hitag µ(micro)\n"
                  );
 
     char div_str[70] = {0};
@@ -272,12 +274,12 @@ int CmdLFCommandRead(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, false);
     uint32_t delay = arg_get_u32_def(ctx, 1, 0);
 
-    int cmd_len = 128;
     char cmd[128] = {0};
+    int cmd_len = sizeof(cmd) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 2, (uint8_t *)cmd, &cmd_len);
 
-    int extra_arg_len = 250;
     char extra_arg[250] = {0};
+    int extra_arg_len = sizeof(extra_arg) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 3, (uint8_t *)extra_arg, &extra_arg_len);
 
     uint16_t period_1 = arg_get_u32_def(ctx, 4, 0);
@@ -324,6 +326,10 @@ int CmdLFCommandRead(const char *Cmd) {
         uint8_t i;
         for (i = 0; i < cmd_len; i++) {
             if ((cmd[i] != '0') && (cmd[i] != '1')) {
+                // avoid include 'W0S' in crc
+                crc_init_ref(&crc, 8, 0x1d, 0xff, 0, false, false);
+                n = 0;
+                data = 0;
                 continue;
             }
             data <<= 1;
@@ -345,14 +351,14 @@ int CmdLFCommandRead(const char *Cmd) {
         }
     }
 
-    memcpy(payload.data, cmd, cmd_len);
+    memcpy(payload.data, cmd, cmd_len + 1);
 
     // extra symbol definition
     uint8_t index_extra = 0;
     int i = 0;
     for (; i < extra_arg_len;) {
 
-        if (index_extra < LF_CMDREAD_MAX_EXTRA_SYMBOLS - 1) {
+        if (index_extra < LF_CMDREAD_MAX_EXTRA_SYMBOLS) {
             payload.symbol_extra[index_extra] = extra_arg[i];
             int tmp = atoi(extra_arg + (i + 1));
             payload.period_extra[index_extra] = tmp;
@@ -362,14 +368,15 @@ int CmdLFCommandRead(const char *Cmd) {
                 i++;
 
         } else {
-            PrintAndLogEx(WARNING, "Too many extra symbols, please define up to %i symbols", LF_CMDREAD_MAX_EXTRA_SYMBOLS);
+            PrintAndLogEx(ERR, "Too many extra symbols, please define up to %i symbols", LF_CMDREAD_MAX_EXTRA_SYMBOLS);
+            return PM3_EINVARG;
         }
     }
 
     // bitbang mode
     if (payload.delay == 0) {
         if (payload.period_0 < 7 || payload.period_1 < 7) {
-            PrintAndLogEx(WARNING, "periods cannot be less than 7us in bit bang mode");
+            PrintAndLogEx(ERR, "periods cannot be less than 7us in bit bang mode");
             return PM3_EINVARG;
         }
     }
@@ -403,7 +410,7 @@ int CmdLFCommandRead(const char *Cmd) {
     int ret = PM3_SUCCESS;
     do {
         clearCommandBuffer();
-        SendCommandNG(CMD_LF_MOD_THEN_ACQ_RAW_ADC, (uint8_t *)&payload, PAYLOAD_HEADER_SIZE + cmd_len);
+        SendCommandNG(CMD_LF_MOD_THEN_ACQ_RAW_ADC, (uint8_t *)&payload, PAYLOAD_HEADER_SIZE + cmd_len + 1);
 
         PacketResponseNG resp;
         // init to ZERO
@@ -534,7 +541,7 @@ int CmdFlexdemod(const char *Cmd) {
 *  this function will save a copy of the current lf config value, and set config to default values.
 *
 */
-int lf_config_savereset(sample_config *config) {
+int lf_resetconfig(sample_config *config) {
 
     if (config == NULL) {
         return PM3_EINVARG;
@@ -558,7 +565,7 @@ int lf_config_savereset(sample_config *config) {
         .verbose = false,
     };
 
-    res = lf_config(&def_config);
+    res = lf_setconfig(&def_config);
     if (res != PM3_SUCCESS) {
         PrintAndLogEx(ERR, "failed to reset LF configuration to default values");
         return res;
@@ -588,7 +595,7 @@ int lf_getconfig(sample_config *config) {
     return PM3_SUCCESS;
 }
 
-int lf_config(sample_config *config) {
+int lf_setconfig(sample_config *config) {
     if (!g_session.pm3_present) return PM3_ENOTTY;
 
     clearCommandBuffer();
@@ -649,7 +656,7 @@ int CmdLFConfig(const char *Cmd) {
 
     // if called with no params, just print the device config
     if (strlen(Cmd) == 0) {
-        return lf_config(NULL);
+        return lf_setconfig(NULL);
     }
 
     if (use_125 + use_134 > 1) {
@@ -722,7 +729,7 @@ int CmdLFConfig(const char *Cmd) {
         config.trigger_threshold = 0;
     }
 
-    return lf_config(&config);
+    return lf_setconfig(&config);
 }
 
 static int lf_read_internal(bool realtime, bool verbose, uint64_t samples) {
@@ -1546,7 +1553,7 @@ static bool check_chiptype(bool getDeviceData) {
 
     //check for em4x05/em4x69 chips first
     uint32_t word = 0;
-    if (em4x05_isblock0(&word)) {
+    if (IfPm3EM4x50() && em4x05_isblock0(&word)) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x05 / EM4x69"));
         PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x05`") " commands");
         retval = true;
@@ -1563,7 +1570,7 @@ static bool check_chiptype(bool getDeviceData) {
 
 #if !defined ICOPYX
     // check for em4x50 chips
-    if (detect_4x50_block()) {
+    if (IfPm3EM4x50() && detect_4x50_block()) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x50"));
         PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x50`") " commands");
         retval = true;
@@ -1571,7 +1578,7 @@ static bool check_chiptype(bool getDeviceData) {
     }
 
     // check for em4x70 chips
-    if (detect_4x70_block()) {
+    if (IfPm3EM4x70() && detect_4x70_block()) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x70"));
         PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x70`") " commands");
         retval = true;
