@@ -380,7 +380,7 @@ static command_t CommandTable[] = {
     {"help",   CmdHelp,            AlwaysAvailable, "This help"},
     {"demod",  CmdGallagherDemod,  AlwaysAvailable, "demodulate an GALLAGHER tag from the GraphBuffer"},
     {"reader", CmdGallagherReader, IfPm3Lf,         "attempt to read and extract tag data"},
-    {"clone",  CmdGallagherClone,  IfPm3Lf,         "clone GALLAGHER tag to T55x7"},
+    {"clone",  CmdGallagherClone,  IfPm3Lf,         "clone GALLAGHER tag to T55x7, Q5/T5555 or EM4305/4469"},
     {"sim",    CmdGallagherSim,    IfPm3Lf,         "simulate GALLAGHER tag"},
     {NULL, NULL, NULL, NULL}
 };
