@@ -21,7 +21,6 @@
 #include "comms.h"
 #include "cmdtrace.h"
 #include "commonutil.h"
-#include "hitag.h"
 #include "fileutils.h"  // savefile
 #include "protocols.h"  // defines
 #include "cliparser.h"
@@ -32,6 +31,7 @@
 #include "pm3_cmd.h"    // return codes
 #include "hitag2/hitag2_crypto.h"
 #include "util_posix.h"             // msclock
+#include "cmdlfhitaghts.h"
 
 static int CmdHelp(const char *Cmd);
 
@@ -308,7 +308,7 @@ static void print_hitag2_configuration(uint32_t uid, uint8_t config) {
             PrintAndLogEx(SUCCESS, "  %s", sprint_breakdown_bin(C_NONE, bs, 8, 5, 2, "Public mode C, Coding: biphase"));
             break;
         case 3:
-            PrintAndLogEx(SUCCESS, "  %s", sprint_breakdown_bin(C_NONE, bs, 8, 5, 2, "Hitag2"));
+            PrintAndLogEx(SUCCESS, "  %s", sprint_breakdown_bin(C_NONE, bs, 8, 5, 2, "Hitag 2"));
             break;
     }
 
@@ -522,6 +522,13 @@ bool hitag2_get_plain(uint8_t *plain,  uint8_t *plen) {
     return false;
 }
 
+// HITAG 2 commands
+#define HITAG2_BINSTR_START_AUTH           "11000"         // get UID and/or start the authentication process
+#define HITAG2_BINSTR_READ_PAGE            "11"            // read page after auth
+#define HITAG2_BINSTR_READ_PAGE_INVERTED   "01"            // as read page but all bits inverted
+#define HITAG2_BINSTR_WRITE_PAGE           "10"            // write page after auth
+#define HITAG2_BINSTR_HALT                 "00"            // silence currently authenticated tag
+
 static uint8_t hitag2_get_page(const char *bs) {
     if ((memcmp(bs + 2, "000", 3) == 0) && (memcmp(bs + 2 + 3 + 2, "111", 3) == 0)) {
         return 0;
@@ -578,24 +585,24 @@ void hitag2_annotate_plain(char *exp, size_t size, const uint8_t *cmd, uint8_t c
             break;
         }
         case 10: {
-            if (memcmp(binstr, HITAG2_HALT, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_HALT, 2) == 0) {
                 snprintf(exp, size, " ");
                 break;
             }
 
             uint8_t page = hitag2_get_page(binstr);
 
-            if (memcmp(binstr, HITAG2_READ_PAGE, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_READ_PAGE, 2) == 0) {
                 snprintf(exp, size, "READ PAGE (" _MAGENTA_("%u") ")", page);
                 break;
             }
 
-            if (memcmp(binstr, HITAG2_READ_PAGE_INVERTED, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_READ_PAGE_INVERTED, 2) == 0) {
                 snprintf(exp, size, "READ PAGE INV (" _MAGENTA_("%u") ")", page);
                 break;
             }
 
-            if (memcmp(binstr, HITAG2_WRITE_PAGE, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_WRITE_PAGE, 2) == 0) {
                 snprintf(exp, size, "WRITE PAGE (" _MAGENTA_("%u") ")", page);
                 break;
             }
@@ -654,7 +661,7 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
         case 5: {
             annotateHitag2_init();
 
-            if (memcmp(binstr, HITAG2_START_AUTH, 5) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_START_AUTH, 5) == 0) {
                 snprintf(exp, size, "START AUTH");
                 _ht2state.state = STATE_START_AUTH;
             } else {
@@ -669,7 +676,7 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
                 break;
             }
 
-            if (memcmp(binstr, HITAG2_HALT, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_HALT, 2) == 0) {
                 snprintf(exp, size, "HALT");
                 _ht2state.state = STATE_HALT;
                 break;
@@ -677,17 +684,17 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
 
             uint8_t page = hitag2_get_page(binstr);
 
-            if (memcmp(binstr, HITAG2_READ_PAGE, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_READ_PAGE, 2) == 0) {
                 snprintf(exp, size, "READ PAGE (" _MAGENTA_("%u") ")", page);
                 break;
             }
 
-            if (memcmp(binstr, HITAG2_READ_PAGE_INVERTED, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_READ_PAGE_INVERTED, 2) == 0) {
                 snprintf(exp, size, "READ PAGE INV (" _MAGENTA_("%u") ")", page);
                 break;
             }
 
-            if (memcmp(binstr, HITAG2_WRITE_PAGE, 2) == 0) {
+            if (memcmp(binstr, HITAG2_BINSTR_WRITE_PAGE, 2) == 0) {
                 snprintf(exp, size, "WRITE PAGE (" _MAGENTA_("%u") ")", page);
                 break;
             }
@@ -815,7 +822,7 @@ static bool getHitag2Uid(uint32_t *uid) {
 static int CmdLFHitagInfo(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag info",
-                  "Hitag2 tag information",
+                  "Hitag 2 tag information",
                   "lf hitag info"
                  );
 
@@ -858,7 +865,7 @@ static int CmdLFHitagInfo(const char *Cmd) {
 static int CmdLFHitagReader(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag reader",
-                  "Act as a Hitag2 reader.  Look for Hitag2 tags until Enter or the pm3 button is pressed\n",
+                  "Act as a Hitag 2 reader.  Look for Hitag 2 tags until Enter or the pm3 button is pressed\n",
                   "lf hitag reader\n"
                   "lf hitag reader -@   -> Continuous mode"
                  );
@@ -891,17 +898,17 @@ static int CmdLFHitagRd(const char *Cmd) {
 
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag read",
-                  "Read Hitag memory. It support HitagS and Hitag 2\n\n"
+                  "Read Hitag memory. It support Hitag S and Hitag 2\n\n"
                   "  Password mode:\n"
                   "    - default key 4D494B52 (MIKR)\n\n"
                   "  Crypto mode: \n"
                   "    - key format ISK high + ISK low\n"
                   "    - default key 4F4E4D494B52 (ONMIKR)\n"
                   ,
-                  "  lf hitag read --hts                         -> HitagS, plain mode\n"
-                  "  lf hitag read --hts --nrar 0102030411223344 -> HitagS, challenge mode\n"
-                  "  lf hitag read --hts --crypto                -> HitagS, crypto mode, def key\n"
-                  "  lf hitag read --hts -k 4F4E4D494B52         -> HitagS, crypto mode\n\n"
+                  "  lf hitag read --hts                         -> Hitag S, plain mode\n"
+                  "  lf hitag read --hts --nrar 0102030411223344 -> Hitag S, challenge mode\n"
+                  "  lf hitag read --hts --crypto                -> Hitag S, crypto mode, def key\n"
+                  "  lf hitag read --hts -k 4F4E4D494B52         -> Hitag S, crypto mode\n\n"
                   "  lf hitag read --ht2 --pwd                   -> Hitag 2, pwd mode, def key\n"
                   "  lf hitag read --ht2 -k 4D494B52             -> Hitag 2, pwd mode\n"
                   "  lf hitag read --ht2 --nrar 0102030411223344 -> Hitag 2, challenge mode\n"
@@ -1293,7 +1300,14 @@ static int CmdLFHitagWriter(const char *Cmd) {
     lf_hitag_data_t packet;
     memset(&packet, 0, sizeof(packet));
 
-    if (use_hts && use_nrar) {
+    if (use_hts && use_plain) {
+        packet.cmd = WHTSF_PLAIN;
+        packet.page = page;
+        memcpy(packet.data, data, sizeof(data));
+
+        PrintAndLogEx(INFO, "Write to " _YELLOW_("Hitag S") " in Plain mode");
+
+    } else if (use_hts && use_nrar) {
         packet.cmd = WHTSF_CHALLENGE;
         memcpy(packet.NrAr, nrar, sizeof(packet.NrAr));
         memcpy(packet.data, data, sizeof(data));
@@ -2018,7 +2032,7 @@ static int CmdLFHitag2Lookup(const char *Cmd) {
 
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag lookup",
-                  "This command take sniffed trace data and try to recovery a Hitag2 crypto key.\n"
+                  "This command take sniffed trace data and try to recovery a Hitag 2 crypto key.\n"
                   " You can either\n"
                   " - verify that NR/AR matches a known crypto key\n"
                   " - verify if NR/AR matches a known 6 byte crypto key in a dictionary",
@@ -2232,7 +2246,7 @@ static int CmdLFHitag2Lookup(const char *Cmd) {
 static int CmdLFHitag2Crack2(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag crack2",
-                  "This command tries to recover 2048 bits of Hitag2 crypto stream data.\n",
+                  "This command tries to recover 2048 bits of Hitag 2 crypto stream data.\n",
                   "lf hitag crack2 --nrar 73AA5A62EAB8529C"
                  );
 
@@ -2439,7 +2453,7 @@ static int CmdLFHitag2Selftest(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, true);
     CLIParserFree(ctx);
 
-    PrintAndLogEx(INFO, "======== " _CYAN_("Hitag2 crypto test") " ============================");
+    PrintAndLogEx(INFO, "======== " _CYAN_("Hitag 2 crypto test") " ============================");
     uint64_t test = hitag2_verify_crypto_test();
     PrintAndLogEx(INFO, "Crypto self test ( %s )", test ? _GREEN_("ok") : _RED_("fail"));
 
@@ -2447,13 +2461,13 @@ static int CmdLFHitag2Selftest(const char *Cmd) {
     PrintAndLogEx(INFO, "Crypto self test ROUND ( %s )", test ? _GREEN_("ok") : _RED_("fail"));
 
     test |= hitag2_benchtest(1);
-    PrintAndLogEx(INFO, "Hitag2 crypto, init + gen 32 bits ( us %" PRIu64 " )", test);
+    PrintAndLogEx(INFO, "Hitag 2 crypto, init + gen 32 bits ( us %" PRIu64 " )", test);
 
     test |= hitag2_benchtest_gen32();
-    PrintAndLogEx(INFO, "Hitag2 crypto, gen new 32 bits only ( us: %" PRIu64 " )", test);
+    PrintAndLogEx(INFO, "Hitag 2 crypto, gen new 32 bits only ( us: %" PRIu64 " )", test);
 
     test |= hitag2_benchtest(1000);
-    PrintAndLogEx(INFO, "Hitag2 crypto, init + gen 32 bits, x1000 ( us: %" PRIu64 " )", test);
+    PrintAndLogEx(INFO, "Hitag 2 crypto, init + gen 32 bits, x1000 ( us: %" PRIu64 " )", test);
 
     PrintAndLogEx(INFO, "--------------------------------------------------------");
     PrintAndLogEx(SUCCESS, "Tests ( %s )", (test) ? _GREEN_("ok") : _RED_("fail"));
@@ -2464,9 +2478,10 @@ static int CmdLFHitag2Selftest(const char *Cmd) {
 static command_t CommandTable[] = {
     {"help",        CmdHelp,                    AlwaysAvailable, "This help"},
     {"list",        CmdLFHitagList,             AlwaysAvailable, "List Hitag trace history"},
+    {"hts",           CmdLFHitagS,                AlwaysAvailable, "{ Hitag S/8211 operations }"},
     {"-----------", CmdHelp,                    IfPm3Hitag,      "------------------------ " _CYAN_("General") " ------------------------"},
     {"info",        CmdLFHitagInfo,             IfPm3Hitag,      "Hitag 2 tag information"},
-    {"reader",      CmdLFHitagReader,           IfPm3Hitag,      "Act line an Hitag 2 reader"},
+    {"reader",      CmdLFHitagReader,           IfPm3Hitag,      "Act like a Hitag 2 reader"},
     {"test",        CmdLFHitag2Selftest,        AlwaysAvailable, "Perform self tests"},
     {"-----------", CmdHelp,                    IfPm3Hitag,      "----------------------- " _CYAN_("Operations") " -----------------------"},
 //    {"demod",       CmdLFHitag2PWMDemod,        IfPm3Hitag,      "PWM Hitag 2 reader message demodulation"},
