@@ -189,8 +189,8 @@ static int CmdPrescoClone(const char *Cmd) {
     uint8_t hex[4] = {0, 0, 0, 0};
     CLIGetHexWithReturn(ctx, 1, hex, &hex_len);
 
-    uint8_t idstr[11];
-    int slen = 9;
+    uint8_t idstr[10];
+    int slen = sizeof(idstr) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     memset(idstr, 0x00, sizeof(idstr));
     CLIGetStrWithReturn(ctx, 2, idstr, &slen);
 
@@ -292,8 +292,8 @@ static int CmdPrescoSim(const char *Cmd) {
     uint8_t hex[4] = {0, 0, 0, 0};
     CLIGetHexWithReturn(ctx, 1, hex, &hex_len);
 
-    uint8_t idstr[11];
-    int slen = 9;
+    uint8_t idstr[10] = {0};
+    int slen = sizeof(idstr) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     memset(idstr, 0x00, sizeof(idstr));
     CLIGetStrWithReturn(ctx, 2, idstr, &slen);
     CLIParserFree(ctx);
