@@ -242,7 +242,7 @@ static command_t CommandTable[] = {
     {"help",    CmdHelp,        AlwaysAvailable, "This help"},
     {"demod",   CmdVikingDemod, AlwaysAvailable, "demodulate a Viking tag from the GraphBuffer"},
     {"reader",  CmdVikingReader,  IfPm3Lf,       "attempt to read and extract tag data"},
-    {"clone",   CmdVikingClone, IfPm3Lf,         "clone Viking tag to T55x7 or Q5/T5555"},
+    {"clone",   CmdVikingClone, IfPm3Lf,         "clone Viking tag to T55x7, Q5/T5555 or EM4305/4469"},
     {"sim",     CmdVikingSim,   IfPm3Lf,         "simulate Viking tag"},
     {NULL, NULL, NULL, NULL}
 };
