@@ -1386,7 +1386,13 @@ int CmdTraceList(const char *Cmd) {
     if (use_buffer == false) {
         download_trace();
     } else if (gs_traceLen == 0 || gs_trace == NULL) {
-        PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace.");
+
+        if (IfPm3Present() == false) {
+            PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace.");
+        } else {
+            PrintAndLogEx(FAILED, "You requested a trace list but there is no trace.");
+        }
+
         PrintAndLogEx(FAILED, "Consider using `" _YELLOW_("trace load") "` or removing parameter `" _YELLOW_("-1") "`");
         return PM3_EINVARG;
     }
@@ -1448,7 +1454,7 @@ int CmdTraceList(const char *Cmd) {
         }
 
         if (protocol == ISO_7816_4)
-            PrintAndLogEx(INFO, _YELLOW_("ISO7816-4 / Smartcard") " - Timings N/A");
+            PrintAndLogEx(INFO, _YELLOW_("ISO7816-4 / Smartcard") " - Timings n/a");
 
         if (protocol == PROTO_HITAG1 || protocol == PROTO_HITAG2 || protocol == PROTO_HITAGS) {
             PrintAndLogEx(INFO, _YELLOW_("Hitag1 / Hitag2 / HitagS") " - Timings in ETU (8us)");
