@@ -42,6 +42,7 @@
 #include "mbedtls/sha1.h"       // SHA1
 #include "cmdhf14a.h"
 #include "gen4.h"
+#include "parity.h"
 
 int mfDarkside(uint8_t blockno, uint8_t key_type, uint64_t *key) {
     uint32_t uid = 0;
@@ -56,6 +57,15 @@ int mfDarkside(uint8_t blockno, uint8_t key_type, uint64_t *key) {
 
     while (true) {
         clearCommandBuffer();
+
+        //TODO: Not really stopping the command in time.
+        //flush queue
+        while (kbd_enter_pressed()) {
+            SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
+            PrintAndLogEx(WARNING, "Aborted via keyboard");
+            return PM3_EOPABORTED;
+        }
+
         struct {
             uint8_t first_run;
             uint8_t blockno;
@@ -66,12 +76,6 @@ int mfDarkside(uint8_t blockno, uint8_t key_type, uint64_t *key) {
         payload.key_type = key_type;
         SendCommandNG(CMD_HF_MIFARE_READER, (uint8_t *)&payload, sizeof(payload));
 
-        //flush queue
-        while (kbd_enter_pressed()) {
-            SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
-            return PM3_EOPABORTED;
-        }
-
         PrintAndLogEx(NORMAL, "");
         PrintAndLogEx(INFO, "Running darkside " NOLF);
 
@@ -79,16 +83,17 @@ int mfDarkside(uint8_t blockno, uint8_t key_type, uint64_t *key) {
         while (true) {
             PrintAndLogEx(NORMAL, "." NOLF);
 
+            if (IsCommunicationThreadDead()) return PM3_EIO;
+
+            //TODO: Not really stopping the command in time.
             if (kbd_enter_pressed()) {
                 SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
+                PrintAndLogEx(WARNING, "\nAborted via keyboard");
                 return PM3_EOPABORTED;
             }
 
             PacketResponseNG resp;
             if (WaitForResponseTimeout(CMD_HF_MIFARE_READER, &resp, 2000)) {
-                if (resp.status == PM3_EOPABORTED) {
-                    return resp.status;
-                }
 
                 struct p {
                     int32_t isOK;
@@ -100,17 +105,36 @@ int mfDarkside(uint8_t blockno, uint8_t key_type, uint64_t *key) {
                     uint8_t ar[4];
                 } PACKED;
 
-                struct p *package = (struct p *) resp.data.asBytes;
+                struct p *package = (struct p *)resp.data.asBytes;
 
-                if (package->isOK == -6) {
-                    *key = 0101;
-                    return 1;
+                if (resp.status != PM3_SUCCESS) {
+                    PrintAndLogEx(NORMAL, "");
+
+                    switch (package->isOK) {
+                        case 2:
+                            PrintAndLogEx(FAILED, "Card is not vulnerable to Darkside attack (doesn't send NACK on authentication requests).");
+                            break;
+                        case 3:
+                            PrintAndLogEx(FAILED, "Card is not vulnerable to Darkside attack (its random number generator is not predictable).");
+                            break;
+                        case 4:
+                            PrintAndLogEx(FAILED, "Card is not vulnerable to Darkside attack (its random number generator seems to be based on the wellknown");
+                            PrintAndLogEx(FAILED, "generating polynomial with 16 effective bits only, but shows unexpected behaviour.");
+                            break;
+                        case 5:
+                            PrintAndLogEx(WARNING, "Button pressed. aborted");
+                            break;
+                        case 6:
+                            *key = 0101;
+                            return PM3_SUCCESS;
+                        default:
+                            PrintAndLogEx(FAILED, "Unknown error. Darkside attack failed.");
+                            break;
+                    }
+
+                    return resp.status;
                 }
 
-                if (package->isOK < 0)
-                    return package->isOK;
-
-
                 uid = (uint32_t)bytes_to_num(package->cuid, sizeof(package->cuid));
                 nt = (uint32_t)bytes_to_num(package->nt, sizeof(package->nr));
                 par_list = bytes_to_num(package->par_list, sizeof(package->par_list));
@@ -197,7 +221,7 @@ int mfCheckKeys(uint8_t blockNo, uint8_t keyType, bool clear_trace, uint8_t keyc
     SendCommandNG(CMD_HF_MIFARE_CHKKEYS, data, (5 + 6 * keycnt));
 
     PacketResponseNG resp;
-    if (!WaitForResponseTimeout(CMD_HF_MIFARE_CHKKEYS, &resp, 2500)) {
+    if (WaitForResponseTimeout(CMD_HF_MIFARE_CHKKEYS, &resp, 2500) == false) {
         return PM3_ETIMEOUT;
     }
     if (resp.status != PM3_SUCCESS) {
@@ -209,11 +233,13 @@ int mfCheckKeys(uint8_t blockNo, uint8_t keyType, bool clear_trace, uint8_t keyc
         bool found;
     } PACKED;
     struct kr *keyresult = (struct kr *)&resp.data.asBytes;
-    if (!keyresult->found) {
+    if (keyresult->found == false) {
         return PM3_ESOFT;
     }
 
-    *key = bytes_to_num(keyresult->key, sizeof(keyresult->key));
+    if (key) {
+        *key = bytes_to_num(keyresult->key, sizeof(keyresult->key));
+    }
     return PM3_SUCCESS;
 }
 
@@ -221,21 +247,24 @@ int mfCheckKeys(uint8_t blockNo, uint8_t keyType, bool clear_trace, uint8_t keyc
 // 0 == ok all keys found
 // 1 ==
 // 2 == Time-out, aborting
-int mfCheckKeys_fast(uint8_t sectorsCnt, uint8_t firstChunk, uint8_t lastChunk, uint8_t strategy,
-                     uint32_t size, uint8_t *keyBlock, sector_t *e_sector, bool use_flashmemory, bool verbose) {
+int mfCheckKeys_fast_ex(uint8_t sectorsCnt, uint8_t firstChunk, uint8_t lastChunk, uint8_t strategy,
+                        uint32_t size, uint8_t *keyBlock, sector_t *e_sector, bool use_flashmemory,
+                        bool verbose, bool quiet, uint16_t singleSectorParams) {
 
     uint64_t t2 = msclock();
 
     // send keychunk
     clearCommandBuffer();
-    SendCommandOLD(CMD_HF_MIFARE_CHKKEYS_FAST, (sectorsCnt | (firstChunk << 8) | (lastChunk << 12)), ((use_flashmemory << 8) | strategy), size, keyBlock, 6 * size);
+    SendCommandOLD(CMD_HF_MIFARE_CHKKEYS_FAST, (sectorsCnt | (firstChunk << 8) | (lastChunk << 12) | (singleSectorParams << 16)), ((use_flashmemory << 8) | strategy), size, keyBlock, 6 * size);
     PacketResponseNG resp;
 
     uint32_t timeout = 0;
     while (WaitForResponseTimeout(CMD_ACK, &resp, 2000) == false) {
 
-        PrintAndLogEx((timeout) ? NORMAL : INFO, "." NOLF);
-        fflush(stdout);
+        if (! quiet) {
+            PrintAndLogEx((timeout) ? NORMAL : INFO, "." NOLF);
+            fflush(stdout);
+        }
 
         timeout++;
 
@@ -249,13 +278,22 @@ int mfCheckKeys_fast(uint8_t sectorsCnt, uint8_t firstChunk, uint8_t lastChunk,
     }
     t2 = msclock() - t2;
 
-    if (timeout) {
+    if (timeout && ! quiet) {
         PrintAndLogEx(NORMAL, "");
     }
 
     // time to convert the returned data.
     uint8_t curr_keys = resp.oldarg[0];
 
+    if ((singleSectorParams >> 15) & 1) {
+        if (curr_keys) {
+            uint64_t foo = bytes_to_num(resp.data.asBytes, 6);
+            PrintAndLogEx(NORMAL, "");
+            PrintAndLogEx(SUCCESS, _GREEN_("Key %s for block %2i found: %012" PRIx64), (singleSectorParams >> 8) & 1 ? "B" : "A", singleSectorParams & 0xFF, foo);
+            return PM3_SUCCESS;
+        }
+    }
+
     if (verbose) {
         PrintAndLogEx(INFO, "Chunk %.1fs | found %u/%u keys (%u)", (float)(t2 / 1000.0), curr_keys, (sectorsCnt << 1), size);
     }
@@ -317,6 +355,11 @@ int mfCheckKeys_fast(uint8_t sectorsCnt, uint8_t firstChunk, uint8_t lastChunk,
     return PM3_ESOFT;
 }
 
+int mfCheckKeys_fast(uint8_t sectorsCnt, uint8_t firstChunk, uint8_t lastChunk, uint8_t strategy,
+                     uint32_t size, uint8_t *keyBlock, sector_t *e_sector, bool use_flashmemory, bool verbose) {
+    return mfCheckKeys_fast_ex(sectorsCnt, firstChunk, lastChunk, strategy, size, keyBlock, e_sector, use_flashmemory, verbose, false, 0);
+}
+
 // Trigger device to use a binary file on flash mem as keylist for mfCheckKeys.
 // As of now,  255 keys possible in the file
 // 6 * 255 = 1500 bytes
@@ -585,11 +628,21 @@ int mfnested(uint8_t blockNo, uint8_t keyType, uint8_t *key, uint8_t trgBlockNo,
             free(statelists[1].head.slhead);
             num_to_bytes(key64, 6, resultKey);
 
-            PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %c -- found valid key [ " _GREEN_("%s") " ]",
-                          package->block,
-                          package->keytype ? 'B' : 'A',
-                          sprint_hex_inrow(resultKey, 6)
-                         );
+            if (package->keytype < 2) {
+                PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %c -- found valid key [ " _GREEN_("%s") " ]",
+                              package->block,
+                              package->keytype ? 'B' : 'A',
+                              sprint_hex_inrow(resultKey, 6)
+                             );
+            } else {
+                PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %02x -- found valid key [ " _GREEN_("%s") " ]",
+                              package->block,
+                              MIFARE_AUTH_KEYA + package->keytype,
+                              sprint_hex_inrow(resultKey, 6)
+                             );
+            }
+
+
             return PM3_SUCCESS;
         }
 
@@ -598,11 +651,17 @@ int mfnested(uint8_t blockNo, uint8_t keyType, uint8_t *key, uint8_t trgBlockNo,
     }
 
 out:
-    PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %c",
-                  package->block,
-                  package->keytype ? 'B' : 'A'
-                 );
-
+    if (package->keytype < 2) {
+        PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %c",
+                      package->block,
+                      package->keytype ? 'B' : 'A'
+                     );
+    } else {
+        PrintAndLogEx(SUCCESS, "\nTarget block %4u key type %02x",
+                      package->block,
+                      MIFARE_AUTH_KEYA + package->keytype
+                     );
+    }
     free(statelists[0].head.slhead);
     free(statelists[1].head.slhead);
     return PM3_ESOFT;
@@ -1371,12 +1430,22 @@ returns:
 2  = cmd failed
 3  = has encrypted nonce
 */
-int detect_classic_static_encrypted_nonce(uint8_t block_no, uint8_t key_type, uint8_t *key) {
+int detect_classic_static_encrypted_nonce_ex(uint8_t block_no, uint8_t key_type, uint8_t *key, uint8_t block_no_nested, uint8_t key_type_nested, uint8_t *key_nested, uint8_t nr_nested, bool reset, bool hardreset, bool addread, bool addauth, bool incblk2, bool corruptnrar, bool corruptnrarparity, bool verbose) {
     clearCommandBuffer();
-    uint8_t cdata[1 + 1 + MIFARE_KEY_SIZE] = { 0 };
+    uint8_t cdata[1 + 1 + MIFARE_KEY_SIZE + 1 + 1 + MIFARE_KEY_SIZE + 1 + 1 + 1 + 1 + 1 + 1 + 1] = { 0 };
     cdata[0] = block_no;
     cdata[1] = key_type;
     memcpy(&cdata[2], key, MIFARE_KEY_SIZE);
+    cdata[8] = block_no_nested;
+    cdata[9] = key_type_nested;
+    memcpy(&cdata[10], key_nested, MIFARE_KEY_SIZE);
+    cdata[16] = nr_nested;
+    cdata[17] = hardreset << 1 | reset;
+    cdata[18] = addread;
+    cdata[19] = addauth;
+    cdata[20] = incblk2;
+    cdata[21] = corruptnrar;
+    cdata[22] = corruptnrarparity;
     SendCommandNG(CMD_HF_MIFARE_STATIC_ENCRYPTED_NONCE, cdata, sizeof(cdata));
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_HF_MIFARE_STATIC_ENCRYPTED_NONCE, &resp, 1000)) {
@@ -1384,11 +1453,64 @@ int detect_classic_static_encrypted_nonce(uint8_t block_no, uint8_t key_type, ui
         if (resp.status == PM3_ESOFT) {
             return NONCE_FAIL;
         }
+        if (verbose && (resp.data.asBytes[0] == NONCE_STATIC_ENC)) {
+            uint32_t uid = resp.data.asBytes[1] << 24 |
+                           resp.data.asBytes[2] << 16 |
+                           resp.data.asBytes[3] << 8 |
+                           resp.data.asBytes[4];
+            uint32_t nt = resp.data.asBytes[5] << 24 |
+                          resp.data.asBytes[6] << 16 |
+                          resp.data.asBytes[7] << 8 |
+                          resp.data.asBytes[8];
+            uint32_t ntenc = resp.data.asBytes[9] << 24 |
+                             resp.data.asBytes[10] << 16 |
+                             resp.data.asBytes[11] << 8 |
+                             resp.data.asBytes[12];
+            uint8_t ntencparenc = resp.data.asBytes[13];
+
+            // recompute nt on client, just because
+            struct Crypto1State mpcs = {0, 0};
+            struct Crypto1State *pcs;
+            pcs = &mpcs;
+            uint64_t ui64key = bytes_to_num(key_nested, 6);
+            crypto1_init(pcs, ui64key); // key_nested
+            uint32_t ks = crypto1_word(pcs, ntenc ^ uid, 1);
+            uint32_t mynt = ks ^ ntenc;
+            if (mynt != nt) {
+                PrintAndLogEx(ERR, "Client computed nT " _YELLOW_("%08x") " does not match ARM computed nT " _YELLOW_("%08x"), mynt, nt);
+            }
+            ntencparenc >>= 4;
+            // [...] Additionally, the bit of keystream used to encrypt the parity bits is reused to encrypt the next bit of plaintext.
+            // we can decrypt first 3 parity bits, not last one as it's using future keystream
+            uint8_t ksp = (((ks >> 16) & 1) << 3) | (((ks >> 8) & 1) << 2) | (((ks >> 0) & 1) << 1);
+            uint8_t ntencpar = ntencparenc ^ ksp;
+            if (validate_prng_nonce(nt)) {
+                PrintAndLogEx(INFO, "nTenc " _GREEN_("%08x") " par {" _YELLOW_("%i%i%i%i") "}=" _YELLOW_("%i%i%ix") " | ks "  _GREEN_("%08x") " | nT " _GREEN_("%08x") " par " _YELLOW_("%i%i%i%i")" | lfsr16 index " _GREEN_("%i"),
+                              ntenc,
+                              (ntencparenc >> 3) & 1, (ntencparenc >> 2) & 1, (ntencparenc >> 1) & 1, ntencparenc & 1,
+                              (ntencpar >> 3) & 1, (ntencpar >> 2) & 1, (ntencpar >> 1) & 1,
+                              ks, nt,
+                              oddparity8((nt >> 24) & 0xFF), oddparity8((nt >> 16) & 0xFF), oddparity8((nt >> 8) & 0xFF), oddparity8(nt & 0xFF),
+                              nonce_distance(0, nt));
+            } else {
+                PrintAndLogEx(INFO, "nTenc " _GREEN_("%08x") " par {" _YELLOW_("%i%i%i%i") "}=" _YELLOW_("%i%i%ix") " | ks "  _YELLOW_("%08x") " | nT " _YELLOW_("%08x") " par " _YELLOW_("%i%i%i%i") " | " _RED_("not lfsr16") " (wrong key)",
+                              ntenc,
+                              (ntencparenc >> 3) & 1, (ntencparenc >> 2) & 1, (ntencparenc >> 1) & 1, ntencparenc & 1,
+                              (ntencpar >> 3) & 1, (ntencpar >> 2) & 1, (ntencpar >> 1) & 1,
+                              ks, nt,
+                              oddparity8((nt >> 24) & 0xFF), oddparity8((nt >> 16) & 0xFF), oddparity8((nt >> 8) & 0xFF), oddparity8(nt & 0xFF)
+                             );
+            }
+        }
         return resp.data.asBytes[0];
     }
     return NONCE_FAIL;
 }
 
+int detect_classic_static_encrypted_nonce(uint8_t block_no, uint8_t key_type, uint8_t *key) {
+    return detect_classic_static_encrypted_nonce_ex(block_no, key_type, key, block_no, key_type, key, 3, false, false, false, false, false, false, false, false);
+}
+
 // try to see if card responses to "Chinese magic backdoor" commands.
 // returns flag
 uint16_t detect_mf_magic(bool is_mfc, uint8_t key_type, uint64_t key) {
@@ -1410,6 +1532,10 @@ uint16_t detect_mf_magic(bool is_mfc, uint8_t key_type, uint64_t key) {
         isMagic = MemLeToUint2byte(resp.data.asBytes);
     }
 
+    if (isMagic) {
+        PrintAndLogEx(NORMAL, "");
+    }
+
     if ((isMagic & MAGIC_FLAG_GEN_1A) == MAGIC_FLAG_GEN_1A) {
         PrintAndLogEx(SUCCESS, "Magic capabilities... " _GREEN_("Gen 1a"));
     }
