@@ -279,6 +279,8 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf iclass chk" },
     { 1, "hf iclass loclass" },
     { 1, "hf iclass lookup" },
+    { 0, "hf iclass legrec" },
+    { 1, "hf iclass legbrute" },
     { 0, "hf iclass sim" },
     { 0, "hf iclass eload" },
     { 0, "hf iclass esave" },
@@ -335,6 +337,7 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf mf help" },
     { 1, "hf mf list" },
     { 0, "hf mf info" },
+    { 0, "hf mf isen" },
     { 0, "hf mf darkside" },
     { 0, "hf mf nested" },
     { 1, "hf mf hardnested" },
@@ -666,6 +669,10 @@ const static vocabulary_t vocabulary[] = {
     { 0, "lf hitag chk" },
     { 1, "lf hitag lookup" },
     { 0, "lf hitag ta" },
+    { 1, "lf hitag hts help" },
+    { 1, "lf hitag hts list" },
+    { 0, "lf hitag hts read" },
+    { 0, "lf hitag hts write" },
     { 1, "lf idteck help" },
     { 1, "lf idteck demod" },
     { 0, "lf idteck reader" },
