@@ -44,12 +44,12 @@
 #endif
 
 
-static int mainret = PM3_ESOFT;
+static int mainret = PM3_SUCCESS;
 
 #ifndef LIBPM3
 #define BANNERMSG1 ""
 #define BANNERMSG2 "   [ :coffee: ]"
-#define BANNERMSG3 "Release v4.18589 - Aurora"
+#define BANNERMSG3 "Release v4.18994 - Backdoor"
 
 typedef enum LogoMode { UTF8, ANSI, ASCII } LogoMode;
 
@@ -553,7 +553,7 @@ check_script:
             if (cmd[0] != '\0') {
                 uint8_t old_printAndLog = g_printAndLog;
                 if (!printprompt) {
-                    g_printAndLog &= PRINTANDLOG_LOG;
+                    g_printAndLog &= ~PRINTANDLOG_PRINT;
                 }
                 char prompt[PROXPROMPT_MAX_SIZE] = {0};
                 prompt_compose(prompt, sizeof(prompt), prompt_ctx, prompt_dev, prompt_net, true);
@@ -1462,6 +1462,8 @@ int main(int argc, char *argv[]) {
         preferences_save();
     }
 
+    free_grabber();
+
     return mainret;
 }
 #endif //LIBPM3
