@@ -44,6 +44,7 @@
 
 #include "comms.h"
 #include "ui.h"
+#include "util_posix.h" // msleep
 
 // Taken from https://github.com/unbit/uwsgi/commit/b608eb1772641d525bfde268fe9d6d8d0d5efde7
 #ifndef SOL_TCP
@@ -358,7 +359,13 @@ serial_port uart_open(const char *pcPortName, uint32_t speed, bool slient) {
 
     free(prefix);
 
-    sp->fd = open(pcPortName, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK);
+    // Freshly available port can take a while before getting permission to access it. Up to 600ms on my machine...
+    for (uint8_t i = 0; i < 10; i++) {
+        sp->fd = open(pcPortName, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK);
+        if (sp->fd != -1 || errno != EACCES)
+            break;
+        msleep(100);
+    }
     if (sp->fd == -1) {
         uart_close(sp);
         return INVALID_SERIAL_PORT;
@@ -387,11 +394,15 @@ serial_port uart_open(const char *pcPortName, uint32_t speed, bool slient) {
         return INVALID_SERIAL_PORT;
     }
 
+    // Flush all lingering data that may exist
+    tcflush(sp->fd, TCIOFLUSH);
+
     // Duplicate the (old) terminal info struct
     sp->tiNew = sp->tiOld;
 
-    // Configure the serial port
-    sp->tiNew.c_cflag = CS8 | CLOCAL | CREAD;
+    // Configure the serial port.
+    // fix:  default to 115200 here seems to fix the white dongle issue. Will need to check proxbuilds later.
+    sp->tiNew.c_cflag = B115200 | CS8 | CLOCAL | CREAD;
     sp->tiNew.c_iflag = IGNPAR;
     sp->tiNew.c_oflag = 0;
     sp->tiNew.c_lflag = 0;
@@ -401,6 +412,17 @@ serial_port uart_open(const char *pcPortName, uint32_t speed, bool slient) {
     // Block until a timer expires (n * 100 mSec.)
     sp->tiNew.c_cc[VTIME] = 0;
 
+    // more configurations
+    sp->tiNew.c_cc[VINTR]    = 0;     /* Ctrl-c */
+    sp->tiNew.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
+    sp->tiNew.c_cc[VERASE]   = 0;     /* del */
+    sp->tiNew.c_cc[VKILL]    = 0;     /* @ */
+    sp->tiNew.c_cc[VEOF]     = 4;     /* Ctrl-d */
+    sp->tiNew.c_cc[VSTART]   = 0;     /* Ctrl-q */
+    sp->tiNew.c_cc[VSTOP]    = 0;     /* Ctrl-s */
+    sp->tiNew.c_cc[VSUSP]    = 0;     /* Ctrl-z */
+    sp->tiNew.c_cc[VEOL]     = 0;     /* '\0' */
+
     // Try to set the new terminal info struct
     if (tcsetattr(sp->fd, TCSANOW, &sp->tiNew) == -1) {
         PrintAndLogEx(ERR, "error: UART set terminal info attribute");
@@ -695,9 +717,14 @@ bool uart_set_speed(serial_port sp, const uint32_t uiPortSpeed) {
     // Set port speed (Input and Output)
     cfsetispeed(&ti, stPortSpeed);
     cfsetospeed(&ti, stPortSpeed);
+
+    // flush
+    tcflush(spu->fd, TCIOFLUSH);
+
     bool result = tcsetattr(spu->fd, TCSANOW, &ti) != -1;
-    if (result)
+    if (result) {
         g_conn.uart_speed = uiPortSpeed;
+    }
     return result;
 }
 
