@@ -36,8 +36,10 @@
 #define UTIL_BUFFER_SIZE_SPRINT 8196
 // global client debug variable
 uint8_t g_debugMode = 0;
-// global client disable logging variable
+// global client enable/disable printing/logging/grabbing variable
 uint8_t g_printAndLog = PRINTANDLOG_PRINT | PRINTANDLOG_LOG;
+// global pointer to grabbed output
+grabbed_output g_grabbed_output = {NULL, 0, 0};
 // global client tell if a pending prompt is present
 bool g_pendingPrompt = false;
 // global CPU core count override
@@ -1145,7 +1147,7 @@ void binstr_2_bytes(uint8_t *target, size_t *targetlen, const char *src) {
     }
 }
 
-void hex_xor(uint8_t *d, uint8_t *x, int n) {
+void hex_xor(uint8_t *d, const uint8_t *x, int n) {
     while (n--) {
         d[n] ^= x[n];
     }
@@ -1161,7 +1163,7 @@ uint8_t GetParity(const uint8_t *bits, uint8_t type, int length) {
 }
 
 // add HID parity to binary array: EVEN prefix for 1st half of ID, ODD suffix for 2nd half
-void wiegand_add_parity(uint8_t *target, uint8_t *source, uint8_t length) {
+void wiegand_add_parity(uint8_t *target, const uint8_t *source, uint8_t length) {
     *(target++) = GetParity(source, EVEN, length / 2);
     memcpy(target, source, length);
     target += length;
@@ -1169,7 +1171,7 @@ void wiegand_add_parity(uint8_t *target, uint8_t *source, uint8_t length) {
 }
 
 // add HID parity to binary array: ODD prefix for 1st half of ID, EVEN suffix for 2nd half
-void wiegand_add_parity_swapped(uint8_t *target, uint8_t *source, uint8_t length) {
+void wiegand_add_parity_swapped(uint8_t *target, const uint8_t *source, uint8_t length) {
     *(target++) = GetParity(source, ODD, length / 2);
     memcpy(target, source, length);
     target += length;
