@@ -422,7 +422,7 @@ void Uint8byteToMemBe(uint8_t *data, uint64_t value) {
     data[7] = (uint8_t)((value >> 0) & 0xffu);
 }
 
-// RotateLeft - Ultralight, Desfire
+// Rotate Left - Ultralight, Desfire
 void rol(uint8_t *data, const size_t len) {
     uint8_t first = data[0];
     for (size_t i = 0; i < len - 1; i++) {
@@ -431,6 +431,18 @@ void rol(uint8_t *data, const size_t len) {
     data[len - 1] = first;
 }
 
+// Rotate Right - Ultralight, Desfire
+void ror(uint8_t *data, const size_t len) {
+    uint8_t last = data[len - 1];
+
+    for (int i = len - 1; i > 0; i--) {
+        data[i] = data[i - 1];
+    }
+
+    data[0] = last;
+}
+
+
 void lsl(uint8_t *data, size_t len) {
     for (size_t n = 0; n < len - 1; n++) {
         data[n] = (data[n] << 1) | (data[n + 1] >> 7);
@@ -537,3 +549,17 @@ bool hexstr_to_byte_array(const char *hexstr, uint8_t *d, size_t *n) {
     }
     return true;
 }
+
+void reverse_arraybytes(uint8_t *arr, size_t len) {
+    size_t i;
+    for (i = 0; i < len ; i++) {
+        arr[i] = reflect8(arr[i]);
+    }
+}
+
+void reverse_arraybytes_copy(uint8_t *arr, uint8_t *dest, size_t len) {
+    size_t i;
+    for (i = 0; i < len ; i++) {
+        dest[i] = reflect8(arr[i]);
+    }
+}
