@@ -114,6 +114,8 @@ void Uint8byteToMemBe(uint8_t *data, uint64_t value);
 
 // rotate left byte array
 void rol(uint8_t *data, const size_t len);
+void ror(uint8_t *data, const size_t len);
+
 void lsl(uint8_t *data, size_t len);
 uint32_t le24toh(const uint8_t data[3]);
 void htole24(uint32_t val, uint8_t data[3]);
@@ -128,4 +130,7 @@ void reverse_array(uint8_t *d, size_t n);
 void reverse_array_copy(const uint8_t *src, int src_len, uint8_t *dest);
 
 bool hexstr_to_byte_array(const char *hexstr, uint8_t *d, size_t *n);
+
+void reverse_arraybytes(uint8_t *arr, size_t len);
+void reverse_arraybytes_copy(uint8_t *arr, uint8_t *dest, size_t len);
 #endif
