@@ -582,7 +582,7 @@ typedef struct {
 
 #define CMD_LF_SNIFF_RAW_ADC                                              0x0360
 
-// For Hitag2 transponders
+// For Hitag 2 transponders
 #define CMD_LF_HITAG_SNIFF                                                0x0370
 #define CMD_LF_HITAG_SIMULATE                                             0x0371
 #define CMD_LF_HITAG_READER                                               0x0372
@@ -590,7 +590,7 @@ typedef struct {
 #define CMD_LF_HITAG2_CRACK                                               0x0378
 #define CMD_LF_HITAG2_CRACK_2                                             0x0379
 
-// For HitagS
+// For Hitag S
 #define CMD_LF_HITAGS_TEST_TRACES                                         0x0367
 #define CMD_LF_HITAGS_SIMULATE                                            0x0368
 #define CMD_LF_HITAGS_READ                                                0x0373
@@ -630,6 +630,7 @@ typedef struct {
 #define CMD_HF_ICLASS_CHKKEYS                                             0x039A
 #define CMD_HF_ICLASS_RESTORE                                             0x039B
 #define CMD_HF_ICLASS_CREDIT_EPURSE                                       0x039C
+#define CMD_HF_ICLASS_RECOVER                                             0x039D
 
 
 // For ISO1092 / FeliCa
@@ -676,6 +677,7 @@ typedef struct {
 #define CMD_HF_MIFARE_ACQ_NONCES                                          0x0614
 #define CMD_HF_MIFARE_STATIC_NESTED                                       0x0615
 #define CMD_HF_MIFARE_STATIC_ENC                                          0x0616
+#define CMD_HF_MIFARE_ACQ_STATIC_ENCRYPTED_NONCES                         0x0617
 
 #define CMD_HF_MIFARE_READBL                                              0x0620
 #define CMD_HF_MIFARE_READBL_EX                                           0x0628
@@ -792,11 +794,7 @@ typedef struct {
 #define FLAG_ANSI        0x08
 
 // Error codes                          Usages:
-
-// Success, regular quit
-#define PM3_SQUIT               2
-// Success, transfer nonces            pm3:        Sending nonces back to client
-#define PM3_SNONCES             1
+// NOTE: Positive values should be reserved for commands in case they need to return multiple statuses and error codes simultaneously.
 // Success (no error)
 #define PM3_SUCCESS             0
 
@@ -865,7 +863,8 @@ typedef struct {
 #define PM3_ENODATA           -98
 // Quit program                         client:     reserved, order to quit the program
 #define PM3_EFATAL            -99
-
+// Regular quit
+#define PM3_SQUIT            -100
 
 // LF
 #define LF_FREQ2DIV(f) ((int)(((12000.0 + (f)/2.0)/(f))-1))
@@ -890,11 +889,6 @@ typedef struct {
 # define UART_TCP_LOCAL_CLIENT_RX_TIMEOUT_MS  40
 # define UART_UDP_LOCAL_CLIENT_RX_TIMEOUT_MS  20
 
-// definitions for multiple FPGA config files support
-#define FPGA_BITSTREAM_LF 1
-#define FPGA_BITSTREAM_HF 2
-#define FPGA_BITSTREAM_HF_FELICA 3
-#define FPGA_BITSTREAM_HF_15 4
 
 // CMD_DEVICE_INFO response packet has flags in arg[0], flag definitions:
 /* Whether a bootloader that understands the g_common_area is present */
