@@ -182,9 +182,9 @@ def selftests():
                      "02E3002FCD44BE", "895EE509DE9D98E2FDAC7ADCC976F24B085D73D063986EF59FE260D9BE08D28C"],
          'pk': "041D92163650161A2548D33881C235D0FB2315C2C31A442F23C87ACF14497C0CBA"},
 
-        # TruST25 (ST25TN) - KeyID ?
+        # TruST25 (ST25TN) - KeyID 0x05 (b31-b24 of block 45)
         # curve=secp128r1, hash=sha256 - from block 52 in ST25TN, followed by ascii UID
-        {'name': "ST25TN512/01K TruST25 (ST) / KeyID ?",
+        {'name': "ST25TN512/01K TruST25 (ST) / KeyID 0x05",
          'samples': ["020339A5940000", "A5E968CEDD7278C46F0FF7ECABAD649C229BCA444915D307E69C1945FA95C9C6",
                      "02643AFD04A000", "0938D86193C603E1B30B17C8117A930205CAC1A8CE88F0EA269FCE2A44244D7B"],
          'pk': "0440004F974F7C76BC8718E523D85FA7B354A9A992BFA966CB8219242F9D274FD6"},
